<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
	version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns=""
>
	<xsl:output 
		method="text" 
		encoding="utf-8" 
		omit-xml-declaration="yes"
		indent="no"/>

<!--~~~~~~~~~~~~
~~~~ constants
~~~~~~~~~~~~~~-->
	<xsl:variable name="UNBOUNDED_MULTIPLICITY">32767</xsl:variable>
	<xsl:variable name="VOID_RETURN">void</xsl:variable>
	<xsl:variable name="DEFAULT_SUBSET_NAME"><![CDATA[{subset}]]></xsl:variable>
	
<!--~~~~~~~~~~~~
~~~	processing
~~~~~~~~~~~~~~-->
	
<!-- root -->
	<xsl:template match="/">
		<!-- comment -->
		<xsl:text><![CDATA[%%% Translation of conceptual class diagram: ]]></xsl:text>
		<xsl:value-of select="ccdDiagram/name"/>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<xsl:apply-templates/>
		<xsl:apply-templates select="ccdDiagram" mode="CWA"/>
		<xsl:apply-templates select="ccdDiagram" mode="UNA"/>
	</xsl:template>
	
	<xsl:template match="ccdDiagram/name">
	</xsl:template>
	
<!-- association -->
	<xsl:template match="association">
		<!-- comment -->
		<![CDATA[%%%	[Association] ]]><xsl:value-of select="name"/>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<!-- comment -->
		<![CDATA[%%%	>	Typing ]]>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<!-- "all" statement --><xsl:for-each select="roles/role"><![CDATA[all X]]><xsl:value-of select="position()"/><xsl:text> </xsl:text></xsl:for-each><![CDATA[(]]>
		<!-- preamble --><xsl:value-of select="@identifierName"/><![CDATA[(]]><xsl:for-each select="roles/role"><xsl:if test="position() &gt; 1"><xsl:text>, </xsl:text></xsl:if><![CDATA[X]]><xsl:value-of select="position()"/></xsl:for-each><![CDATA[)]]>
		<!-- implies --><![CDATA[ -> ]]>
		<!-- aftermath --><xsl:for-each select="roles/role"><xsl:if test="position() &gt; 1"><xsl:text><![CDATA[ & ]]></xsl:text></xsl:if><xsl:value-of select="participantClass"/><![CDATA[(X]]><xsl:value-of select="position()"/><![CDATA[)]]></xsl:for-each>
		<![CDATA[).]]>
		<xsl:for-each select="roles/role">
			<xsl:if test="multiplicity/@minimum &gt; 0 or multiplicity/@maximum &lt; $UNBOUNDED_MULTIPLICITY">
				<xsl:variable name="participantClassIdName">
					<xsl:value-of select="participantClass"/>
				</xsl:variable>
				<!-- comment -->
				<xsl:text><![CDATA[%%%	>	]]></xsl:text>
				<xsl:value-of select="ancestor::association/../*[@identifierName = $participantClassIdName]/name"/>
				<xsl:text> role</xsl:text>
				<xsl:if test="name">
					<xsl:text><![CDATA[ (]]></xsl:text>
					<xsl:value-of select="name"/>
					<xsl:text><![CDATA[)]]></xsl:text>
				</xsl:if>					
				<![CDATA[%%%]]>
				<!-- end of comment -->
				<!-- comment -->
				<![CDATA[%%%	>	>	Multiplicity {]]><xsl:value-of select="multiplicity/@minimum"/><![CDATA[..]]><xsl:call-template name="write_maximum_multiplicity"><xsl:with-param name="maximumMultiplicity"><xsl:value-of select="multiplicity/@maximum"/></xsl:with-param></xsl:call-template><![CDATA[} constraints ]]>				
				<![CDATA[%%%]]>
				<!-- end of comment -->
			</xsl:if>
			<xsl:if test="multiplicity/@minimum &gt; 0">
			<!-- OF COURSE the association arity MUST be 2 (see "T.Mancini - Progettazione del Software 1 - slides - S.A.1", p.58) -->
				<xsl:variable name="myPosition"><xsl:value-of select="position()"/></xsl:variable>
				<!-- comment -->
				<![CDATA[%%%	>	>	>	Minimum multiplicity: ]]><xsl:value-of select="multiplicity/@minimum"/>
				<![CDATA[%%%]]>
				<!-- end of comment -->
				<!-- "all" statement --><![CDATA[all X (]]>
				<!-- preamble --><xsl:value-of select="../role[position() != $myPosition]/participantClass"/><![CDATA[(X)]]>
				<!-- implies --><![CDATA[ -> ]]><!-- aftermath --><xsl:call-template name="exists_zN_to_1">
					<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@minimum"/></xsl:with-param>
				</xsl:call-template>
				<xsl-text><![CDATA[(]]><!-- \n -->
				</xsl-text>
				<xsl:call-template name="verifies_predicate_with_zN_to_1">
					<xsl:with-param name="predicateStringPrefix"><xsl:value-of select="../../@identifierName"/><![CDATA[(]]><xsl:for-each select="../role[position() &lt; $myPosition ]"><![CDATA[X, ]]></xsl:for-each></xsl:with-param>
					<xsl:with-param name="predicateStringSuffix"><xsl:for-each select="../role[position() &gt; $myPosition ]"><![CDATA[, X]]></xsl:for-each><![CDATA[)]]></xsl:with-param>
					<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@minimum"/></xsl:with-param>
					<xsl:with-param name="connective"><![CDATA[&]]></xsl:with-param>
				</xsl:call-template>
				<xsl:if test="multiplicity/@minimum &gt; 1">
					<![CDATA[&]]>
				</xsl:if>
				<xsl:call-template name="zN_to_1_comparing_to_zN-1_to_1">
					<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@minimum"/></xsl:with-param>
					<xsl:with-param name="comparisonSymbol"><![CDATA[!=]]></xsl:with-param>
					<xsl:with-param name="connective"><![CDATA[&]]></xsl:with-param>
				</xsl:call-template>
				<![CDATA[)]]>
				<![CDATA[).]]>
			</xsl:if>
			<xsl:if test="multiplicity/@maximum &lt; $UNBOUNDED_MULTIPLICITY">
			<!-- OF COURSE the association arity MUST be 2 (see "T.Mancini - Progettazione del Software 1 - slides - S.A.1", p.58) -->
				<xsl:variable name="myPosition"><xsl:value-of select="position()"/></xsl:variable>
				<!-- comment -->
				<![CDATA[%%%	>	>	>	Maximum multiplicity: ]]><xsl:value-of select="multiplicity/@maximum"/>
				<![CDATA[%%%]]>
				<!-- end of comment -->
				<!-- "all" statement -->
				<![CDATA[all X ]]><xsl:call-template name="all_zN_to_1">
					<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@maximum + 1"/></xsl:with-param>
				</xsl:call-template>
				<xsl:text><![CDATA[( ]]><!-- \n -->
					<![CDATA[( ]]><!-- \n -->
				</xsl:text>
				<!-- preamble -->
				<xsl:value-of select="../role[position() != $myPosition]/participantClass"/>
				<xsl:text><![CDATA[(X)]]></xsl:text>
				<![CDATA[ & ]]>
				<xsl:call-template name="verifies_predicate_with_zN_to_1">
					<xsl:with-param name="predicateStringPrefix"><xsl:value-of select="../../@identifierName"/><![CDATA[(]]><xsl:for-each select="../role[position() &lt; $myPosition ]"><![CDATA[X, ]]></xsl:for-each></xsl:with-param>
					<xsl:with-param name="predicateStringSuffix"><xsl:for-each select="../role[position() &gt; $myPosition ]"><![CDATA[, X]]></xsl:for-each><![CDATA[)]]></xsl:with-param>
					<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@maximum + 1"/></xsl:with-param>
					<xsl:with-param name="connective"><![CDATA[&]]></xsl:with-param>
				</xsl:call-template>
				<xsl:text><!-- \n -->
					<![CDATA[)]]></xsl:text>
				<!-- implies -->
				<xsl:text><![CDATA[ -> ]]></xsl:text>
				<!-- aftermath -->
				<xsl:text><![CDATA[(]]><!-- \n -->
				</xsl:text>
				<xsl:call-template name="zN_to_1_comparing_to_zN-1_to_1">
					<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@maximum + 1"/></xsl:with-param>
					<xsl:with-param name="comparisonSymbol"><![CDATA[=]]></xsl:with-param>
					<xsl:with-param name="connective"><![CDATA[|]]></xsl:with-param>
				</xsl:call-template>
				<![CDATA[)]]>
				<![CDATA[).]]>
			</xsl:if>
		</xsl:for-each>
		<xsl:if test="properties/*">
			<xsl:apply-templates select="properties"/>
		</xsl:if>
	</xsl:template>
	
<!-- generalization -->
	<xsl:template match="generalization">
		<!-- comment -->
		<xsl:variable name="baseClassIdName"><xsl:value-of select="baseClass"/></xsl:variable>
		<xsl:text><![CDATA[%%%	[Generalization] ]]></xsl:text>
		<xsl:if test="name">
			<xsl:value-of select="name"/>
			<xsl-text> : </xsl-text>
		</xsl:if>
		<xsl:for-each select="subClasses/subClass">
			<xsl:variable name="subClassIdName"><xsl:value-of select="."/></xsl:variable>
			<xsl:if test="position() &gt; 1">
				<xsl:text><![CDATA[ + ]]></xsl:text>
			</xsl:if>
			<xsl:value-of select="ancestor::generalization/../*[@identifierName = $subClassIdName]/name"/>
		</xsl:for-each>
		<xsl:text><![CDATA[ < ]]></xsl:text>
		<xsl:value-of select="../*[@identifierName = $baseClassIdName]/name"/>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<xsl:for-each select="subClasses/subClass">
			<!-- "all" statement --><xsl:text><![CDATA[all X ( ]]></xsl:text>
			<!-- preamble --><xsl:value-of select="."/><xsl:text><![CDATA[(X)]]></xsl:text>
			<!-- implies --><xsl:text><![CDATA[ -> ]]></xsl:text>
			<!-- aftermath --><xsl:value-of select="ancestor::generalization/baseClass"/><xsl:text><![CDATA[(X)]]></xsl:text>
			<xsl:text><![CDATA[ ).]]><!-- \n -->
			</xsl:text>
		</xsl:for-each>
		<xsl:if test="@disjoint = 'true'">
			<!-- comment -->
			<![CDATA[%%%	>	Disjointness]]>
			<![CDATA[%%%]]>
			<!-- end of comment -->
			<xsl:apply-templates select="subClasses/subClass" mode="DISJOINTNESS"/>
		</xsl:if>
		<xsl:if test="@complete = 'true'">
			<!-- comment -->
			<![CDATA[%%%	>	Completeness]]>
			<![CDATA[%%%]]>
			<!-- end of comment -->
			<!-- "all" statement --><xsl:text><![CDATA[all X ( ]]></xsl:text>
			<!-- preamble --><xsl:value-of select="baseClass"/><xsl:text><![CDATA[(X)]]></xsl:text>
			<!-- implies --><xsl:text><![CDATA[ -> (]]><!-- \n -->
			</xsl:text>
			<!-- aftermath -->
			<xsl:for-each select="subClasses/subClass">
				<xsl:if test="position() &gt; 1">
					<xsl:text><![CDATA[ |]]><!-- \n -->
					</xsl:text>
				</xsl:if>
				<xsl:value-of select="."/><xsl:text><![CDATA[(X)]]></xsl:text>
			</xsl:for-each>
			<![CDATA[)]]>
			<![CDATA[).]]>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="subClass" mode="DISJOINTNESS">
		<xsl:variable name="currentDisjointSubClass"><xsl:value-of select="."/></xsl:variable>
		<xsl:variable name="currentDisjointSubClassPosition"><xsl:value-of select="position()"/></xsl:variable>
		<xsl:for-each select="../subClass[position() &gt; $currentDisjointSubClassPosition]">
			<!-- "all" statement --><xsl:text><![CDATA[all X ( ]]></xsl:text>
			<!-- preamble --><xsl:value-of select="$currentDisjointSubClass"/><xsl:text>(X)</xsl:text>
			<!-- implies --><xsl:text><![CDATA[ -> ]]></xsl:text>
			<!-- aftermath --><xsl:text><![CDATA[- ]]></xsl:text><xsl:value-of select="."/><xsl:text>(X)</xsl:text><![CDATA[ ).]]>
		</xsl:for-each>
	</xsl:template>
	
<!-- specialization -->
	<xsl:template match="specialization">
		<!-- comment -->
		<xsl:text><![CDATA[%%%	[Specialization] ]]></xsl:text>
		<xsl:if test="name != $DEFAULT_SUBSET_NAME">
			<xsl:value-of select="name"/><xsl:text><![CDATA[ : ]]></xsl:text>
		</xsl:if>
		<xsl:text><![CDATA["]]></xsl:text>
		<xsl:value-of select="../association[@identifierName = current()/source]/name"/>
		<xsl:text><![CDATA[" is a subset of "]]></xsl:text>
		<xsl:value-of select="../association[@identifierName = current()/destination]/name"/>
		<xsl:text><![CDATA["]]></xsl:text>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<!-- "all" statement --><xsl:for-each select="../association[@identifierName = current()/source]/roles/role"><![CDATA[all X]]><xsl:value-of select="position()"/><xsl:text> </xsl:text></xsl:for-each><![CDATA[(]]>
		<!-- preamble --><xsl:value-of select="../association[@identifierName = current()/source]/@identifierName"/><![CDATA[(]]><xsl:for-each select="../association[@identifierName = current()/source]/roles/role"><xsl:if test="position() &gt; 1"><xsl:text>, </xsl:text></xsl:if><![CDATA[X]]><xsl:value-of select="position()"/></xsl:for-each><![CDATA[)]]>
		<!-- implies --><![CDATA[->]]>
		<!-- aftermath --><xsl:value-of select="../association[@identifierName = current()/destination]/@identifierName"/><![CDATA[(]]><xsl:for-each select="../association[@identifierName = current()/source]/roles/role"><xsl:if test="position() &gt; 1"><xsl:text>, </xsl:text></xsl:if><![CDATA[X]]><xsl:value-of select="position()"/></xsl:for-each><![CDATA[)]]>
		<![CDATA[).]]>
	</xsl:template>
	
<!-- dependency -->
	<xsl:template match="dependency"></xsl:template>
	
<!-- class -->
	<xsl:template match="class">
		<xsl:if test="properties/*">
			<!-- comment -->
			<![CDATA[%%%	[Class] ]]><xsl:value-of select="name"/>
			<![CDATA[%%%]]>
			<!-- end of comment -->
			<xsl:apply-templates select="properties"/>
		</xsl:if>
	</xsl:template>
	
<!-- properties -->
	<xsl:template match="properties">
		<!-- comment -->
		<![CDATA[%%%	>	Properties]]>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<xsl:apply-templates select="attribute"/>
		<xsl:apply-templates select="operation"/>
	</xsl:template>
	
<!-- class attribute -->
	<xsl:template match="class/properties/attribute">
		<!-- comment -->
		<xsl:text><![CDATA[%%%	>	>	[Attribute] ]]></xsl:text>
		<xsl:value-of select="name"/>
		<xsl:text><![CDATA[ : ]]></xsl:text>
		<xsl:value-of select="@type"/>
		<xsl:if test="multiplicity/@minimum &lt; 0 or multiplicity/@maximum &gt; 1">
			<xsl:text><![CDATA[ {]]></xsl:text>
			<xsl:value-of select="multiplicity/@minimum"/>
			<xsl:text><![CDATA[..]]></xsl:text><xsl:call-template name="write_maximum_multiplicity">
				<xsl:with-param name="maximumMultiplicity"><xsl:value-of select="multiplicity/@maximum"/></xsl:with-param>
			</xsl:call-template>
			<xsl:text><![CDATA[}]]><!-- \n -->
			</xsl:text>
		</xsl:if>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<!-- comment -->
		<![CDATA[%%%	>	>	>	Typing ]]>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<!-- "all" statement -->
		<xsl:text><![CDATA[all X all Y ( ]]><!-- \n -->
		</xsl:text>
		<!-- preamble -->
		<xsl:value-of select="../../@identifierName"/>
		<xsl:text><![CDATA[(X)]]></xsl:text>
		<xsl:text><![CDATA[ & ]]></xsl:text>
		<xsl:value-of select="@identifierName"/>
		<xsl:text><![CDATA[(X, Y)]]></xsl:text>
		<!-- implies --><![CDATA[ -> ]]>
		<!-- aftermath -->
		<xsl:value-of select="@type"/>
		<xsl:text><![CDATA[(Y)]]></xsl:text>
		<![CDATA[).]]>
		<xsl:if test="multiplicity/@minimum &gt; 0 or multiplicity/@maximum &lt; $UNBOUNDED_MULTIPLICITY">
			<!-- comment -->
			<![CDATA[%%%	>	>	>	Multiplicity {]]><xsl:value-of select="multiplicity/@minimum"/><![CDATA[..]]><xsl:call-template name="write_maximum_multiplicity"><xsl:with-param name="maximumMultiplicity"><xsl:value-of select="multiplicity/@maximum"/></xsl:with-param></xsl:call-template><![CDATA[} constraints ]]>				
			<![CDATA[%%%]]>
			<!-- end of comment -->
		</xsl:if>
		<xsl:if test="multiplicity/@minimum &gt; 0">
			<!-- comment -->
			<![CDATA[%%%	>	>	>	>	Minimum multiplicity: ]]><xsl:value-of select="multiplicity/@minimum"/>
			<![CDATA[%%%]]>
			<!-- end of comment -->
			<!-- "all" statement --><![CDATA[all X (]]>
			<!-- preamble --><xsl:value-of select="../../@identifierName"/><![CDATA[(X)]]>
			<!-- implies --><![CDATA[ -> ]]><!-- aftermath --><xsl:call-template name="exists_zN_to_1">
				<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@minimum"/></xsl:with-param>
			</xsl:call-template>
			<xsl-text><![CDATA[(]]><!-- \n -->
			</xsl-text>
			<xsl:call-template name="verifies_predicate_with_zN_to_1">
				<xsl:with-param name="predicateStringPrefix"><xsl:value-of select="@identifierName"/><![CDATA[(X, ]]></xsl:with-param>
				<xsl:with-param name="predicateStringSuffix"><![CDATA[)]]></xsl:with-param>
				<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@minimum"/></xsl:with-param>
				<xsl:with-param name="connective"><![CDATA[&]]></xsl:with-param>
			</xsl:call-template>
			<xsl:if test="multiplicity/@minimum &gt; 1">
				<![CDATA[&]]>
			</xsl:if>
			<xsl:call-template name="zN_to_1_comparing_to_zN-1_to_1">
				<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@minimum"/></xsl:with-param>
				<xsl:with-param name="comparisonSymbol"><![CDATA[!=]]></xsl:with-param>
				<xsl:with-param name="connective"><![CDATA[&]]></xsl:with-param>
			</xsl:call-template>
			<![CDATA[)]]>
			<![CDATA[).]]>
		</xsl:if>
		<xsl:if test="multiplicity/@maximum &lt; $UNBOUNDED_MULTIPLICITY">
			<!-- comment -->
			<![CDATA[%%%	>	>	>	>	Maximum multiplicity: ]]><xsl:value-of select="multiplicity/@maximum"/>
			<![CDATA[%%%]]>
			<!-- end of comment -->
			<!-- "all" statement -->
			<![CDATA[all X ]]><xsl:call-template name="all_zN_to_1">
				<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@maximum + 1"/></xsl:with-param>
			</xsl:call-template>
			<xsl:text><![CDATA[( ]]><!-- \n -->
				<![CDATA[( ]]><!-- \n -->
			</xsl:text>
			<!-- preamble -->
			<xsl:value-of select="../../@identifierName"/>
			<xsl:text><![CDATA[(X)]]></xsl:text>
			<![CDATA[ & ]]>
			<xsl:call-template name="verifies_predicate_with_zN_to_1">
				<xsl:with-param name="predicateStringPrefix"><xsl:value-of select="@identifierName"/><![CDATA[(X, ]]></xsl:with-param>
				<xsl:with-param name="predicateStringSuffix"><![CDATA[)]]></xsl:with-param>
				<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@maximum + 1"/></xsl:with-param>
				<xsl:with-param name="connective"><![CDATA[&]]></xsl:with-param>
			</xsl:call-template>
			<xsl:text><!-- \n -->
				<![CDATA[)]]></xsl:text>
			<!-- implies -->
			<xsl:text><![CDATA[ -> ]]></xsl:text>
			<!-- aftermath -->
			<xsl:text><![CDATA[(]]><!-- \n -->
			</xsl:text>
			<xsl:call-template name="zN_to_1_comparing_to_zN-1_to_1">
				<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@maximum + 1"/></xsl:with-param>
				<xsl:with-param name="comparisonSymbol"><![CDATA[=]]></xsl:with-param>
				<xsl:with-param name="connective"><![CDATA[|]]></xsl:with-param>
			</xsl:call-template>
			<![CDATA[)]]>
			<![CDATA[).]]>
		</xsl:if>
	</xsl:template>
	
<!-- association attribute -->
	<xsl:template match="association/properties/attribute">
		<!-- comment -->
		<xsl:text><![CDATA[%%%	>	>	[Attribute] ]]></xsl:text>
		<xsl:value-of select="name"/>
		<xsl:text><![CDATA[ : ]]></xsl:text>
		<xsl:value-of select="@type"/>
		<xsl:if test="multiplicity/@minimum &lt; 0 or multiplicity/@maximum &gt; 1">
			<xsl:text><![CDATA[ {]]></xsl:text>
			<xsl:value-of select="multiplicity/@minimum"/>
			<xsl:text><![CDATA[..]]></xsl:text><xsl:call-template name="write_maximum_multiplicity">
				<xsl:with-param name="maximumMultiplicity"><xsl:value-of select="multiplicity/@maximum"/></xsl:with-param>
			</xsl:call-template>
			<xsl:text><![CDATA[}]]><!-- \n -->
			</xsl:text>
		</xsl:if>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<!-- comment -->
		<![CDATA[%%%	>	>	>	Typing ]]>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<!-- "all" statement -->
		<xsl:text><![CDATA[all Y ]]></xsl:text>
		<xsl:for-each select="ancestor::association/roles/role">
			<xsl:text><![CDATA[all X]]></xsl:text>
			<xsl:value-of select="position()"/>
			<xsl:text> </xsl:text>
		</xsl:for-each>
		<xsl:text><![CDATA[(]]><!-- \n -->
		</xsl:text>
		<!-- preamble -->
		<xsl:value-of select="ancestor::association/@identifierName"/>
		<xsl:text><![CDATA[(]]></xsl:text>
		<xsl:for-each select="ancestor::association/roles/role">
			<xsl:if test="position() &gt; 1">
				<xsl:text><![CDATA[, ]]></xsl:text>
			</xsl:if>
			<xsl:text><![CDATA[X]]></xsl:text>
			<xsl:value-of select="position()"/>
		</xsl:for-each>
		<xsl:text><![CDATA[) & ]]></xsl:text>
		<xsl:value-of select="@identifierName"/>
		<xsl:text><![CDATA[(]]></xsl:text>
		<xsl:for-each select="ancestor::association/roles/role">
			<xsl:if test="position() &gt; 1">
				<xsl:text><![CDATA[, ]]></xsl:text>
			</xsl:if>
			<xsl:text><![CDATA[X]]></xsl:text>
			<xsl:value-of select="position()"/>
		</xsl:for-each>
		<xsl:text><![CDATA[, Y)]]><!-- \n -->
		</xsl:text>
		<!-- implies --><![CDATA[ -> ]]>
		<!-- aftermath -->
		<xsl:for-each select="ancestor::association/roles/role">
			<xsl:if test="position() &gt; 1">
				<xsl:text><![CDATA[ & ]]><!-- \n -->
				</xsl:text>
			</xsl:if>
			<xsl:value-of select="participantClass"/>
			<xsl:text><![CDATA[(X]]></xsl:text>
			<xsl:value-of select="position()"/>
			<xsl:text><![CDATA[)]]></xsl:text>
		</xsl:for-each>
		<xsl:text><![CDATA[ & ]]><!-- \n -->
		</xsl:text>
		<xsl:value-of select="@type"/>
		<xsl:text><![CDATA[(Y)]]></xsl:text>
		<![CDATA[).]]>

		<xsl:if test="multiplicity/@minimum &gt; 0 or multiplicity/@maximum &lt; $UNBOUNDED_MULTIPLICITY">
			<!-- comment -->
			<![CDATA[%%%	>	>	>	Multiplicity {]]><xsl:value-of select="multiplicity/@minimum"/><![CDATA[..]]><xsl:call-template name="write_maximum_multiplicity"><xsl:with-param name="maximumMultiplicity"><xsl:value-of select="multiplicity/@maximum"/></xsl:with-param></xsl:call-template><![CDATA[} constraints ]]>				
			<![CDATA[%%%]]>
			<!-- end of comment -->
		</xsl:if>

		<xsl:if test="multiplicity/@minimum &gt; 0">
			<!-- comment -->
			<![CDATA[%%%	>	>	>	>	Minimum multiplicity: ]]><xsl:value-of select="multiplicity/@minimum"/>
			<![CDATA[%%%]]>
			<!-- end of comment -->
			<!-- "all" statement --><xsl:for-each select="ancestor::association/roles/role"><![CDATA[all X]]><xsl:value-of select="position()"/><xsl:text> </xsl:text></xsl:for-each><![CDATA[(]]>
			<!-- preamble --><xsl:value-of select="ancestor::association/@identifierName"/><![CDATA[(]]><xsl:for-each select="ancestor::association/roles/role"><xsl:if test="position() &gt; 1"><xsl:text>, </xsl:text></xsl:if><![CDATA[X]]><xsl:value-of select="position()"/></xsl:for-each><![CDATA[)]]>
			<!-- implies --><![CDATA[ -> ]]>
			<!-- aftermath -->
			<xsl:call-template name="exists_zN_to_1">
				<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@minimum"/></xsl:with-param>
			</xsl:call-template>
			<xsl-text><![CDATA[(]]><!-- \n -->
			</xsl-text>
			<xsl:call-template name="verifies_predicate_with_zN_to_1">
				<xsl:with-param name="predicateStringPrefix">
					<xsl:value-of select="@identifierName"/>
					<xsl:text><![CDATA[(]]></xsl:text>
					<xsl:for-each select="ancestor::association/roles/role">
						<xsl:text><![CDATA[X]]></xsl:text>
						<xsl:value-of select="position()"/>
						<xsl:text><![CDATA[, ]]></xsl:text>
					</xsl:for-each>
				</xsl:with-param>
				<xsl:with-param name="predicateStringSuffix"><![CDATA[)]]></xsl:with-param>
				<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@minimum"/></xsl:with-param>
				<xsl:with-param name="connective"><![CDATA[&]]></xsl:with-param>
			</xsl:call-template>
			<xsl:if test="multiplicity/@minimum &gt; 1">
				<![CDATA[&]]>
			</xsl:if>
			<xsl:call-template name="zN_to_1_comparing_to_zN-1_to_1">
				<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@minimum"/></xsl:with-param>
				<xsl:with-param name="comparisonSymbol"><![CDATA[!=]]></xsl:with-param>
				<xsl:with-param name="connective"><![CDATA[&]]></xsl:with-param>
			</xsl:call-template>
			<![CDATA[)]]>
			<![CDATA[).]]>
		</xsl:if>
		
		<xsl:if test="multiplicity/@maximum &lt; $UNBOUNDED_MULTIPLICITY">
			<!-- comment -->
			<![CDATA[%%%	>	>	>	>	Maximum multiplicity: ]]><xsl:value-of select="multiplicity/@maximum"/>
			<![CDATA[%%%]]>
			<!-- end of comment -->
			<!-- "all" statement -->
			<xsl:for-each select="ancestor::association/roles/role">
				<xsl:text><![CDATA[all X]]></xsl:text>
				<xsl:value-of select="position()"/>
				<xsl:text> </xsl:text>
			</xsl:for-each>
			<xsl:call-template name="all_zN_to_1">
				<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@maximum + 1"/></xsl:with-param>
			</xsl:call-template>
			<xsl:text><![CDATA[( ]]><!-- \n -->
				<![CDATA[( ]]><!-- \n -->
			</xsl:text>
			<!-- preamble -->
			<xsl:value-of select="ancestor::association/@identifierName"/>
			<xsl:text><![CDATA[(]]></xsl:text>
			<xsl:for-each select="ancestor::association/roles/role">
				<xsl:if test="position() &gt; 1">
					<xsl:text><![CDATA[, ]]></xsl:text>
				</xsl:if>
				<xsl:text><![CDATA[X]]></xsl:text>
				<xsl:value-of select="position()"/>
			</xsl:for-each>
			<xsl:text><![CDATA[) & ]]><!-- \n -->
			</xsl:text>
			<xsl:call-template name="verifies_predicate_with_zN_to_1">
				<xsl:with-param name="predicateStringPrefix">
					<xsl:value-of select="@identifierName"/>
					<xsl:text><![CDATA[(]]></xsl:text>
					<xsl:for-each select="ancestor::association/roles/role">
						<xsl:if test="position() &gt; 1">
							<xsl:text><![CDATA[, ]]></xsl:text>
						</xsl:if>
						<xsl:text><![CDATA[X]]></xsl:text>
						<xsl:value-of select="position()"/>
					</xsl:for-each>
					<xsl:text><![CDATA[, ]]></xsl:text>
				</xsl:with-param>
				<xsl:with-param name="predicateStringSuffix"><![CDATA[)]]></xsl:with-param>
				<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@maximum + 1"/></xsl:with-param>
				<xsl:with-param name="connective"><![CDATA[&]]></xsl:with-param>
			</xsl:call-template>
			<xsl:text><!-- \n -->
				<![CDATA[)]]></xsl:text>
			<!-- implies -->
			<xsl:text><![CDATA[ -> ]]></xsl:text>
			<!-- aftermath -->
			<xsl:text><![CDATA[(]]><!-- \n -->
			</xsl:text>
			<xsl:call-template name="zN_to_1_comparing_to_zN-1_to_1">
				<xsl:with-param name="currentNumber"><xsl:value-of select="multiplicity/@maximum + 1"/></xsl:with-param>
				<xsl:with-param name="comparisonSymbol"><![CDATA[=]]></xsl:with-param>
				<xsl:with-param name="connective"><![CDATA[|]]></xsl:with-param>
			</xsl:call-template>
			<![CDATA[)]]>
			<![CDATA[).]]>
		</xsl:if>
	</xsl:template>
		
<!-- class operation -->
	<xsl:template match="class/properties/operation">
		<!-- comment -->
		<xsl:text><![CDATA[%%%	>	>	[Operation] ]]></xsl:text>
		<xsl:value-of select="name"/>
		<xsl:text><![CDATA[(]]></xsl:text>
		<xsl:for-each select="inputParameters/parameter">
			<xsl:if test="position() &gt; 1">
				<xsl:text><![CDATA[, ]]></xsl:text>
			</xsl:if>
			<xsl:value-of select="name"/>
			<xsl:text><![CDATA[ : ]]></xsl:text>
			<xsl:value-of select="@type"/>
			<xsl:if test="multiplicity/@minimum &lt; 0 or multiplicity/@maximum &gt; 1">
				<xsl:text><![CDATA[ {]]></xsl:text>
				<xsl:value-of select="multiplicity/@minimum"/>
				<xsl:text><![CDATA[..]]></xsl:text><xsl:call-template name="write_maximum_multiplicity">
					<xsl:with-param name="maximumMultiplicity"><xsl:value-of select="multiplicity/@maximum"/></xsl:with-param>
				</xsl:call-template>
				<xsl:text><![CDATA[}]]><!-- \n -->
				</xsl:text>
			</xsl:if>
		</xsl:for-each>
		<xsl:text><![CDATA[)]]></xsl:text>
		<xsl:text><![CDATA[ : ]]></xsl:text>
		<xsl:value-of select="returnTypeString"/>
			<xsl:if test="multiplicity/@minimum &lt; 0 or multiplicity/@maximum &gt; 1">
				<xsl:text><![CDATA[ {]]></xsl:text>
				<xsl:value-of select="multiplicity/@minimum"/>
				<xsl:text><![CDATA[..]]></xsl:text><xsl:call-template name="write_maximum_multiplicity">
					<xsl:with-param name="maximumMultiplicity"><xsl:value-of select="multiplicity/@maximum"/></xsl:with-param>
				</xsl:call-template>
				<xsl:text><![CDATA[}]]><!-- \n -->
				</xsl:text>
			</xsl:if>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<!-- comment -->
		<![CDATA[%%%	>	>	>	Typing ]]>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<!-- "all" statement -->
		<xsl:text><![CDATA[all X ]]></xsl:text>
		<xsl:for-each select="inputParameters/parameter">
			<xsl:text><![CDATA[all P]]></xsl:text>
			<xsl:value-of select="position()"/>
			<xsl:text> </xsl:text>
		</xsl:for-each>
		<xsl:if test="returnTypeString != $VOID_RETURN">
			<xsl:text><![CDATA[all R ]]></xsl:text>
		</xsl:if>
		<xsl:text><![CDATA[(]]><!-- \n -->
		</xsl:text>
		<!-- preamble -->
		<xsl:value-of select="@identifierName"/>
		<xsl:text><![CDATA[(X]]></xsl:text>
		<xsl:for-each select="inputParameters/parameter">
			<xsl:text><![CDATA[, P]]></xsl:text>
			<xsl:value-of select="position()"/>
		</xsl:for-each>
		<xsl:if test="returnTypeString != $VOID_RETURN">
			<xsl:text><![CDATA[, R]]></xsl:text>
		</xsl:if>
		<xsl:text><![CDATA[)]]></xsl:text>
		<!-- implies --><![CDATA[->]]><![CDATA[ (]]>
		<!-- aftermath -->
		<xsl:value-of select="ancestor::class/@identifierName"/>
		<xsl:text><![CDATA[(X)]]></xsl:text>
		<xsl:for-each select="inputParameters/parameter">
			<xsl:text><![CDATA[ &]]><!-- \n -->
			</xsl:text>
			<xsl:value-of select="@type"/>
			<xsl:text><![CDATA[(P]]></xsl:text>
			<xsl:value-of select="position()"/>
			<xsl:text><![CDATA[)]]></xsl:text>
		</xsl:for-each>
		<xsl:if test="returnTypeString != $VOID_RETURN">
			<xsl:text><![CDATA[ &]]><!-- \n -->
			</xsl:text>
			<xsl:value-of select="returnTypeString"/>
			<xsl:text><![CDATA[(R)]]><!-- \n -->
			</xsl:text>
		</xsl:if>
		<![CDATA[)]]>
		<![CDATA[).]]>
		<xsl:if test="returnTypeString != $VOID_RETURN">
			<!-- comment -->
			<![CDATA[%%%	>	>	>	Unique result ]]>
			<![CDATA[%%%]]>
			<!-- end of comment -->
			<!-- "all" statement -->
			<xsl:text><![CDATA[all X ]]></xsl:text>
			<xsl:for-each select="inputParameters/parameter">
				<xsl:text><![CDATA[all P]]></xsl:text>
				<xsl:value-of select="position()"/>
				<xsl:text> </xsl:text>
			</xsl:for-each>
			<xsl:text><![CDATA[all R1 all R2 ]]></xsl:text>
			<xsl:text><![CDATA[(]]><!-- \n -->
			</xsl:text>
			<!-- preamble -->
			<xsl:value-of select="@identifierName"/>
			<xsl:text><![CDATA[(X]]></xsl:text>
			<xsl:for-each select="inputParameters/parameter">
				<xsl:text><![CDATA[, P]]></xsl:text>
				<xsl:value-of select="position()"/>
			</xsl:for-each>
			<xsl:text><![CDATA[, R1) & ]]><!-- \n -->
			</xsl:text>
			<xsl:value-of select="@identifierName"/>
			<xsl:text><![CDATA[(X]]></xsl:text>
			<xsl:for-each select="inputParameters/parameter">
				<xsl:text><![CDATA[, P]]></xsl:text>
				<xsl:value-of select="position()"/>
			</xsl:for-each>
			<xsl:text><![CDATA[, R2)]]></xsl:text>
			<!-- implies --><![CDATA[->]]><![CDATA[ (]]>
			<!-- aftermath -->
			<xsl:text><![CDATA[R1 = R2]]></xsl:text>
			<![CDATA[)]]>
			<![CDATA[).]]>
		</xsl:if>
	</xsl:template>

<!-- Restriction for variables to be an instance, or a value -->
	<xsl:template match="ccdDiagram" mode="CWA">
		<!-- comment -->
		<![CDATA[%%%	Every variable must be an instance, or a value]]>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<![CDATA[all X (]]>
		<xsl:for-each select="//class">
			<xsl:if test="position() &gt; 1">
				<xsl:text><![CDATA[ |]]><!-- \n -->
				</xsl:text>
			</xsl:if>
			<xsl:value-of select="@identifierName"/>
			<xsl:text>(X)</xsl:text>
		</xsl:for-each>
		<xsl:for-each select="//@type[not(preceding::*/@type = .)
				and not(ancestor::ccdDiagram//class/@identifierName = .)]">
			<xsl:text><![CDATA[ |]]><!-- \n -->
			</xsl:text>
			<xsl:value-of select="."/>
			<xsl:text>(X)</xsl:text>
		</xsl:for-each>
		<![CDATA[).]]>
	</xsl:template>

<!-- disjunction among classes not linked by generalizations or hierarchies, among classes and types, among types -->
	<xsl:template match="ccdDiagram" mode="UNA">
		<!-- comment -->
		<![CDATA[%%%	Implicit disjointness]]>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<!-- comment -->
		<![CDATA[%%%	>	Implicit disjointness for classes not linked by hierarchies or generalizations, among them, and with respect to types]]>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<xsl:for-each select="//class[not(/ccdDiagram//generalization/subClasses//subClass = @identifierName)]">
			<xsl:variable name="currentClassPosition"><xsl:value-of select="position()"/></xsl:variable>
			<xsl:variable name="currentClassIdentifierName"><xsl:value-of select="@identifierName"/></xsl:variable>
			<xsl:for-each select="//class[not(/ccdDiagram//generalization/subClasses//subClass = @identifierName)]">
				<xsl:if test="(position() &gt; $currentClassPosition)">
					<!-- "all" statement --><xsl:text><![CDATA[all X (]]></xsl:text>
					<!-- preamble --><xsl:value-of select="$currentClassIdentifierName"/><xsl:text>(X)</xsl:text>
					<!-- implies --><xsl:text><![CDATA[ -> ]]></xsl:text>
					<!-- aftermath --><xsl:text><![CDATA[- ]]></xsl:text><xsl:value-of select="@identifierName"/><xsl:text><![CDATA[(X)]]></xsl:text>
					<xsl:text><![CDATA[).]]><!-- \n -->
					</xsl:text>
				</xsl:if>
			</xsl:for-each>
			<xsl:for-each select="//@type[not(preceding::*/@type = .)
					and not(ancestor::ccdDiagram//class/@identifierName = .)]">
				<!-- "all" statement --><xsl:text><![CDATA[all X (]]></xsl:text>
				<!-- preamble --><xsl:value-of select="$currentClassIdentifierName"/><xsl:text>(X)</xsl:text>
				<!-- implies --><xsl:text><![CDATA[ -> ]]></xsl:text>
				<!-- aftermath --><xsl:text><![CDATA[- ]]></xsl:text><xsl:value-of select="."/><xsl:text><![CDATA[(X)]]></xsl:text>
				<xsl:text><![CDATA[).]]><!-- \n -->
				</xsl:text>
			</xsl:for-each>
		</xsl:for-each>
		<!-- comment -->
		<![CDATA[%%%	>	Implicit disjointness among types]]>
		<![CDATA[%%%]]>
		<!-- end of comment -->
		<xsl:for-each select="//@type[not(preceding::*/@type = .)
					and not(ancestor::ccdDiagram//class/@identifierName = .)]">
			<xsl:variable name="currentTypePosition"><xsl:value-of select="position()"/></xsl:variable>
			<xsl:variable name="currentTypeName"><xsl:value-of select="."/></xsl:variable>
			<xsl:for-each select="//@type[not(preceding::*/@type = .)
						and not(ancestor::ccdDiagram//class/@identifierName = .)]">
				<xsl:if test="position() &gt; $currentTypePosition">
					<!-- "all" statement --><xsl:text><![CDATA[all X (]]></xsl:text>
					<!-- preamble --><xsl:value-of select="$currentTypeName"/><xsl:text>(X)</xsl:text>
					<!-- implies --><xsl:text><![CDATA[ -> ]]></xsl:text>
					<!-- aftermath --><xsl:text><![CDATA[- ]]></xsl:text><xsl:value-of select="."/><xsl:text><![CDATA[(X)]]></xsl:text>
					<xsl:text><![CDATA[).]]><!-- \n -->
					</xsl:text>
				</xsl:if>
			</xsl:for-each>
		</xsl:for-each>
	</xsl:template>
	
<!--~~~~~~~~~~~~
~~~	functions
~~~~~~~~~~~~~~-->
	
	<!-- exists_zN_to_1($currentNumber) -->
	<!--
		Writes, given $currentNumber = 5,
		exists Z5 exists Z4 ... exists Z1
	-->
	<xsl:template name="exists_zN_to_1">
		<xsl:param name="currentNumber"/>
			<![CDATA[exists Z]]><xsl:value-of select="$currentNumber"/>
			<xsl:text> </xsl:text>
			<xsl:if test="$currentNumber &gt; 1">
				<xsl:call-template name="exists_zN_to_1">
					<xsl:with-param name="currentNumber">
						<xsl:value-of select="$currentNumber - 1"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:if>
	</xsl:template>
	
	<!-- all_zN_to_1($currentNumber) -->
	<!--
		Writes, given $currentNumber = 5,
		all Z5 all Z4 ... all Z1
	-->
	<xsl:template name="all_zN_to_1">
		<xsl:param name="currentNumber"/>
			<![CDATA[all Z]]><xsl:value-of select="$currentNumber"/>
			<xsl:text> </xsl:text>
			<xsl:if test="$currentNumber &gt; 1">
				<xsl:call-template name="all_zN_to_1">
					<xsl:with-param name="currentNumber"><xsl:value-of select="$currentNumber - 1"/></xsl:with-param>
				</xsl:call-template>
			</xsl:if>
	</xsl:template>

	<!-- verifies_predicate_with_zN_to_1($predicateStringPrefix, $predicateStringSuffix, $currentNumber) -->
	<!--
		Writes, given $predicateStringPrefix = 'Predicate(X, Y, ', $predicateStringSuffix = ')', $currentNumber = 3, $connective = '&'
		Predicate(X, Y, Z3) & Predicate(X, Y, Z2) & Predicate(X, Y, Z1)
	-->
	<xsl:template name="verifies_predicate_with_zN_to_1">
		<xsl:param name="predicateStringPrefix"/>
		<xsl:param name="predicateStringSuffix"/>
		<xsl:param name="currentNumber"/>
		<xsl:param name="connective"/>
			<xsl:value-of select="$predicateStringPrefix"/><![CDATA[Z]]><xsl:value-of select="$currentNumber"/><xsl:value-of select="$predicateStringSuffix"/>
			<xsl:if test="$currentNumber &gt; 1">
				<xsl:text> </xsl:text>
				<xsl:value-of select="$connective"/>
				<xsl:text> <!-- \n -->
				</xsl:text>
				<xsl:call-template name="verifies_predicate_with_zN_to_1">
					<xsl:with-param name="predicateStringPrefix"><xsl:value-of select="$predicateStringPrefix"/></xsl:with-param>
					<xsl:with-param name="predicateStringSuffix"><xsl:value-of select="$predicateStringSuffix"/></xsl:with-param>
					<xsl:with-param name="currentNumber"><xsl:value-of select="$currentNumber - 1"/></xsl:with-param>
					<xsl:with-param name="connective"><xsl:value-of select="$connective"/></xsl:with-param>
				</xsl:call-template>
			</xsl:if>
	</xsl:template>
	
	<!-- zN_to_1_comparing_to_zN-1_to_1($currentNumber) -->
	<!--
		Writes, given $currentNumber = 5, $comparisonSymbol = '!=', $connective = '&'
		Z5 != Z4 & ... & Z5 != Z1 & Z4 != Z3 & ... & Z4 != Z1 & ... & Z2 != Z1
	-->
	<xsl:template name="zN_to_1_comparing_to_zN-1_to_1">
		<xsl:param name="currentNumber"/>
		<xsl:param name="comparisonSymbol"/>
		<xsl:param name="connective"/>
			<xsl:call-template name="zN_comparing_to_zN-1_to_1">
				<xsl:with-param name="currentNumber"><xsl:value-of select="$currentNumber"/></xsl:with-param>
				<xsl:with-param name="diff">1</xsl:with-param>
				<xsl:with-param name="comparisonSymbol"><xsl:value-of select="$comparisonSymbol"/></xsl:with-param>
				<xsl:with-param name="connective"><xsl:value-of select="$connective"/></xsl:with-param>
			</xsl:call-template>
			<xsl:if test="$currentNumber &gt; 2"><xsl:text> </xsl:text><xsl:value-of select="$connective"/><xsl:text> </xsl:text><xsl:call-template name="zN_to_1_comparing_to_zN-1_to_1">
					<xsl:with-param name="currentNumber"><xsl:value-of select="$currentNumber -1"/></xsl:with-param>
					<xsl:with-param name="comparisonSymbol"><xsl:value-of select="$comparisonSymbol"/></xsl:with-param>
					<xsl:with-param name="connective"><xsl:value-of select="$connective"/></xsl:with-param>
				</xsl:call-template>
			</xsl:if>
	</xsl:template>

	
	<!-- zN_comparing_to_zN-1_to_1($currentNumber, $diff) -->
	<!--
		Writes, given $currentNumber = 6, $diff = 1, $comparisonSymbol = '!=', $connective = '&'
		Z6 != Z5 & Z6 != Z4 & ... & Z6 != Z1
	-->
	<xsl:template name="zN_comparing_to_zN-1_to_1">
		<xsl:param name="currentNumber"/>
		<xsl:param name="diff"/>
		<xsl:param name="comparisonSymbol"/>
		<xsl:param name="connective"/>
			<xsl:if test="$diff &lt; $currentNumber">
				<xsl:call-template name="zN_comparing_to_zM">
					<xsl:with-param name="numberN"><xsl:value-of select="$currentNumber"/></xsl:with-param>
					<xsl:with-param name="numberM"><xsl:value-of select="$currentNumber - $diff"/></xsl:with-param>
					<xsl:with-param name="comparisonSymbol"><xsl:value-of select="$comparisonSymbol"/></xsl:with-param>
				</xsl:call-template>
				<xsl:if test="$diff &lt; $currentNumber -1"><xsl:text> </xsl:text><xsl:value-of select="$connective"/><xsl:text> </xsl:text></xsl:if>
				<xsl:call-template name="zN_comparing_to_zN-1_to_1">
					<xsl:with-param name="currentNumber"><xsl:value-of select="$currentNumber"/></xsl:with-param>
					<xsl:with-param name="diff"><xsl:value-of select="$diff + 1"/></xsl:with-param>
					<xsl:with-param name="comparisonSymbol"><xsl:value-of select="$comparisonSymbol"/></xsl:with-param>
					<xsl:with-param name="connective"><xsl:value-of select="$connective"/></xsl:with-param>
				</xsl:call-template>
			</xsl:if>
	</xsl:template>	
	
	<!-- zN_comparing_to_zM($numberN, $numberM) -->
	<!--
		Writes, given $numberN = 4, $numberM = 3, $comparisonSymbol = '!='
		Z4 != Z3
	-->
	<xsl:template name="zN_comparing_to_zM">
		<xsl:param name="numberN"/>
		<xsl:param name="numberM"/>
		<xsl:param name="comparisonSymbol"/>
			<![CDATA[Z]]><xsl:value-of select="$numberN"/><xsl:text> </xsl:text><xsl:value-of select="$comparisonSymbol"/><xsl:text> </xsl:text><![CDATA[Z]]><xsl:value-of select="$numberM"/>
	</xsl:template>
	
	<!-- write_maximum_multiplicity($maximumMultiplicity) -->
	<!--
		Writes, given $maximumMultiplicity = $UNBOUNDED_MULTIPLICITY,
		*
		In any other case, rewrites the number passed as parameter
	-->
	<xsl:template name="write_maximum_multiplicity">
		<xsl:param name="maximumMultiplicity"/>
			<xsl:choose><xsl:when test="$maximumMultiplicity = $UNBOUNDED_MULTIPLICITY"><![CDATA[*]]></xsl:when><xsl:otherwise><xsl:value-of select="$maximumMultiplicity"/></xsl:otherwise></xsl:choose>
	</xsl:template>

</xsl:stylesheet>