package it.uniroma1.dis.mfis.redia.verifint.control;

import java.io.*;
import java.util.SortedSet;
import java.util.TreeSet;

import it.uniroma1.dis.mfis.redia.verifint.control._exceptions.ApplicationException;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDDiagram;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types.CTDataType;


public class Archive implements Serializable
{
	private static final long serialVersionUID = -4621459021708555168L;
	public static final String FILE_LOADING_EXCEPTION_MESSAGE_TEMPLATE =
		"Error in loading file %1s:\n%2s";
	public static final String FILE_SAVING_EXCEPTION_MESSAGE_TEMPLATE =
		"Error in saving file %1s:\n%2s";
	
	protected static Archive instance;
	private Boolean unsaved;
	public static final String DEFAULT_FILE_SYSTEM_STORAGE_PATH = "temp.mfis";

	private CCDDiagram conceptualClassDiagram;
	private SortedSet<CTDataType> conceptuaDataTypes;

	protected Archive()
	{
		this.conceptualClassDiagram = null;
		this.conceptuaDataTypes = new TreeSet<CTDataType>();
		this.setUnsaved(false);
	}

	void save(String path) throws ApplicationException
	{
		try
		{
			ObjectOutputStream out;
			out = new ObjectOutputStream(new FileOutputStream(path));
			out.writeObject(instance);
			out.close();
			instance.setUnsaved(false);
		}
		catch(IOException err)
		{
			err.printStackTrace(System.err);
			throw new ApplicationException(
					String.format(
							FILE_SAVING_EXCEPTION_MESSAGE_TEMPLATE, path, err
						)
					);
		}
		this.setUnsaved(false);
	}

	void save() throws ApplicationException {
		this.save(DEFAULT_FILE_SYSTEM_STORAGE_PATH);
	}

	boolean load(String path) throws ApplicationException
	{
        ObjectInputStream in;
		try
		{
			in = new ObjectInputStream(new FileInputStream(path));
			instance = (Archive)in.readObject();
			in.close();
			instance.setUnsaved(false);
		}
		catch(FileNotFoundException err)
		{
			return false;
		}
		catch(ClassNotFoundException err)
		{
			err.printStackTrace(System.err);
			throw new ApplicationException(
					String.format(
							FILE_LOADING_EXCEPTION_MESSAGE_TEMPLATE, path, err
						)
					);
		}
		catch(InvalidClassException err)
		{
			err.printStackTrace(System.err);
			throw new ApplicationException(
					String.format(
							FILE_LOADING_EXCEPTION_MESSAGE_TEMPLATE, path, err
						)
					);
		}
		catch(IOException err)
		{
			err.printStackTrace(System.err);
			throw new ApplicationException(
					String.format(
							FILE_LOADING_EXCEPTION_MESSAGE_TEMPLATE, path, err
						)
					);
		}
		return true;
	}

	boolean load() throws ApplicationException {
		return this.load(DEFAULT_FILE_SYSTEM_STORAGE_PATH);
	}

	static Archive getInstance()
	{
		if(instance==null) {
			instance = new Archive();
			return instance;
		}
		return instance;
	}

	void setUnsaved(boolean unsaved) {
		this.unsaved = unsaved;
	}

	boolean isUnsaved() {
		return this.unsaved;
	}

	CCDDiagram getConceptualClassDiagram() {
		return conceptualClassDiagram;
	}

	protected void setConceptualClassDiagram(
			CCDDiagram conceptualClassDiagram) {
		this.conceptualClassDiagram = conceptualClassDiagram;
	}

	SortedSet<CTDataType> getConceptuaDataTypes() {
		return conceptuaDataTypes;
	}

	protected void setConceptuaDataTypes(SortedSet<CTDataType> conceptDataTypes) {
		this.conceptuaDataTypes = conceptDataTypes;
	}
}