package it.uniroma1.dis.mfis.redia.verifint.control;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.util.Properties;

/**
 * <br />It implements the <em>Singleton</em> pattern.
 * @author cdc
 */
public class ExecutionProperties {
	public static final String INI_FILE_LOADING_EXCEPTION_MESSAGE_TEMPLATE =
		"Error in loading ini file:\n" +
		"%1s\n" +
		"%2s";
	
	public static final String INI_FILE_PATH =
		System.getProperty("user.dir") + "/setup.ini";
	private Properties properties;

	private static ExecutionProperties instance;

	private ExecutionProperties() {
		File iniFile = new File(INI_FILE_PATH);
		if (!iniFile.isFile() || !iniFile.canRead()) {
			throw new IllegalArgumentException(
					String.format(
							INI_FILE_LOADING_EXCEPTION_MESSAGE_TEMPLATE,
							INI_FILE_PATH,
							"must be a readable file!")
			);
		}
		this.properties = new Properties();
		try {
			properties.load(new FileInputStream(iniFile));
		} catch (IOException ioe) {
			throw new IllegalArgumentException(
					String.format(
							INI_FILE_LOADING_EXCEPTION_MESSAGE_TEMPLATE,
							INI_FILE_PATH,
							ioe.getMessage())
			);
		}
	}
	
	public static ExecutionProperties getInstance() {
		if (instance == null)
			return new ExecutionProperties();
		return instance;
	}
	
	public String getProperty(String key) {
		return this.properties.getProperty(key);
	}
}