 package it.uniroma1.dis.mfis.redia.verifint.control.io;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;

import it.uniroma1.dis.mfis.redia.verifint.control.ConceptualClassDiagramManager;
import it.uniroma1.dis.mfis.redia.verifint.control.io._interfaces.WidgetsManager;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.Identifiable;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.*;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.operations._interfaces.COpParameter;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types.CTBaseType;
import it.uniroma1.dis.mfis.redia.verifint.view.widgets.Container;
import it.uniroma1.dis.mfis.redia.verifint.view.widgets.Option;
import it.uniroma1.dis.mfis.redia.verifint.view.widgets.SingleChoice;
import it.uniroma1.dis.mfis.redia.verifint.view.widgets.TextField;
import it.uniroma1.dis.mfis.redia.verifint.view.widgets.Widget;
import it.uniroma1.dis.mfis.redia.verifint.view.widgets.IOWidget.WidgetEvent;
import it.uniroma1.dis.mfis.redia.verifint.view.widgets._interfaces.WidgetListener;

public class DCCWidgetsManager
implements WidgetsManager, WidgetListener {
	public static final String GENERALIZATION_LABEL = "Generalization";
	public static final String CLASS_LABEL = "Class";
	public static final String ASSOCIATION_LABEL = "Association";
	public static final String DEPENDENCY_LABEL = "Dependency";
	
	public static final String TYPE_LABEL = ":";
	public static final String PARAMETERS_LABEL = "(Parameters)";
	public static final String ATTRIBUTES_LABEL = "Attributes";
	public static final String OPERATIONS_LABEL = "Operations";
	public static final String MINIMUM_MULTIPLICITY_LABEL = "Multiplicity";
	public static final String MAXIMUM_MULTIPLICITY_LABEL = "..";
	public static final String ROLES_LABEL = "Participant classes";
	
	public static final String NAME_IDENTIFIER = ":name";
	public static final String TYPE_IDENTIFIER = ":type";
	public static final String PARAMETERS_IDENTIFIER = ":parameters";
	public static final String MAXIMUM_MULTIPLICITY_IDENTIFIER =
		":maximumMultiplicity";
	public static final String MINIMUM_MULTIPLICITY_IDENTIFIER =
		":minimumMultiplicity";
	public static final String MULTIPLICITY_IDENTIFIER = ":multiplicity";
	public static final String OPERATIONS_IDENTIFIER = ":operations";
	public static final String ATTRIBUTES_IDENTIFIER = ":attributes";
	
	private ConceptualClassDiagramManager dccMan;

	public DCCWidgetsManager(ConceptualClassDiagramManager dccMan) {
		super();
		this.dccMan = dccMan;
	}

	/**
	 * TODO ricordati di inserire pure i Listener!
	 * TODO ricordati di inserire pure i bottoni (Elimina, Aggiungi)!
	 */
	public Widget buildWidgetsBlock(Identifiable element) {
		Container container = null;
		if (element instanceof CCDAttribute) {
			CCDAttribute attribute = (CCDAttribute)element;

			container = new Container(element.getId().toString(), true);
			TextField attributeNameTextField = new TextField();
			SingleChoice typeChoice = new SingleChoice();
			
			attributeNameTextField.setValue(attribute.getName());
			attributeNameTextField.addWidgetEventInterceptors(this);

			for (CTBaseType baseType : this.dccMan.getBaseTypes().values()) {
				typeChoice.options.add(
						new Option(baseType, baseType.getName()));
			}
			// TODO devi poter scegliere fra tutti i CTDataType			
			
			typeChoice.label = TYPE_LABEL;
			typeChoice.setValue(attribute.getType());

			container.widgets.put(
					NAME_IDENTIFIER, attributeNameTextField);
			container.widgets.put(
					TYPE_IDENTIFIER, typeChoice);
			
			return container;
		}
		else if (element instanceof CCDParameter) {
			CCDParameter parameter = (CCDParameter)element;

			container = new Container(element.getId().toString(), true);
			TextField parameterNameTextField = new TextField();
			SingleChoice typeChoice = new SingleChoice();
			
			parameterNameTextField.setValue(parameter.getName());
			
			for (CTBaseType baseType : this.dccMan.getBaseTypes().values()) {
				typeChoice.options.add(
						new Option(baseType, baseType.getName()));
			}
			// TODO devi poter scegliere fra tutti i CTType

			typeChoice.label = TYPE_LABEL;
			typeChoice.setValue(parameter.getType());
			
			container.widgets.put(
					NAME_IDENTIFIER, parameterNameTextField);
			container.widgets.put(
					TYPE_IDENTIFIER, typeChoice);
		}
		else if (element instanceof CCDOperation) {
			CCDOperation operation = (CCDOperation)element;

			container = new Container(element.getId().toString(), true);
			Container parametersContainer = new Container();
			TextField operationNameTextField = new TextField();
			SingleChoice returnTypeChoice = new SingleChoice();
			
			operationNameTextField.setValue(operation.getName());
			
			for (CTBaseType baseType : this.dccMan.getBaseTypes().values()) {
				returnTypeChoice.options.add(
						new Option(baseType, baseType.getName()));
			}
			// TODO devi poter scegliere fra tutti i CTType
			// TODO devi poter scegliere anche "void"!

			for (COpParameter parameter :
				operation.getInputParameters()) {
				CCDParameter param = (CCDParameter)parameter; 
				parametersContainer.widgets.put(
						param.getId().toString(),
						this.buildWidgetsBlock(param));
			}
			parametersContainer.label = PARAMETERS_LABEL;
			
			returnTypeChoice.label = TYPE_LABEL;
			returnTypeChoice.setValue(operation.getReturnType());
			
			container.widgets.put(
					NAME_IDENTIFIER, operationNameTextField);
			container.widgets.put(
					PARAMETERS_IDENTIFIER, parametersContainer);
			container.widgets.put(
					TYPE_IDENTIFIER, returnTypeChoice);
		}
		else if (element instanceof CCDRole) {
			CCDRole role = (CCDRole)element;

			container = new Container(element.getId().toString());
			container.label = role.getParticipantClass().getName();

			Container multiplicityContainer = new Container(true);
			TextField roleNameTextField = new TextField();
			TextField minimumMultiplicityTextField =
				new TextField();	// TODO filtra l'input del TextField!
			TextField maximumMultiplicityTextField =
				new TextField();	// TODO filtra l'input del TextField!
			
			roleNameTextField.setValue(role.getName());
			minimumMultiplicityTextField.setValue(
					role.getMultiplicity().getMinimum());
			minimumMultiplicityTextField.label =
				MINIMUM_MULTIPLICITY_LABEL;
			maximumMultiplicityTextField.setValue(
					role.getMultiplicity().getMaximum());
			maximumMultiplicityTextField.label =
				MAXIMUM_MULTIPLICITY_LABEL;
			
			multiplicityContainer.widgets.put(
					MINIMUM_MULTIPLICITY_IDENTIFIER,
					minimumMultiplicityTextField);
			multiplicityContainer.widgets.put(
					MAXIMUM_MULTIPLICITY_IDENTIFIER,
					maximumMultiplicityTextField);
			
			container.widgets.put(
					NAME_IDENTIFIER, roleNameTextField);
			container.widgets.put(
					MULTIPLICITY_IDENTIFIER, multiplicityContainer);
		}
		else if (element instanceof CCDClass) {
			CCDClass classElement = (CCDClass)element;

			container = new Container(element.getId().toString());
			container.label = CLASS_LABEL;
			
			Container attributesContainer = new Container();
			Container operationsContainer = new Container();
			TextField classNameTextField = new TextField();
			
			classNameTextField.setValue(classElement.getName());
			
			attributesContainer.label = ATTRIBUTES_LABEL;
			for (CCDAttribute attribute :
				this.getAttributesFromPropertiesSet(
						classElement.getProperties())) {
				attributesContainer.widgets.put(
						attribute.getId().toString(),
						this.buildWidgetsBlock(attribute));
			}
			
			operationsContainer.label = OPERATIONS_LABEL;
			for (CCDOperation operation : this.getOperationsFromPropertiesSet(
						classElement.getProperties())) {
				operationsContainer.widgets.put(
						operation.getId().toString(),
						this.buildWidgetsBlock(operation));
			}
			
			container.widgets.put(
					NAME_IDENTIFIER, classNameTextField);
			container.widgets.put(
					ATTRIBUTES_IDENTIFIER, attributesContainer);
			container.widgets.put(
					OPERATIONS_IDENTIFIER, operationsContainer);
		}
		else if (element instanceof CCDAssociation) {
			CCDAssociation association = (CCDAssociation)element;

			container = new Container(element.getId().toString());
			container.label = ASSOCIATION_LABEL;
			
			Container attributesContainer = new Container();
			Container operationsContainer = new Container();
			Container rolesContainer = new Container();
			
			TextField associationNameTextField = new TextField();
			
			associationNameTextField.setValue(association.getName());
			
			rolesContainer.label = ROLES_LABEL;
			for (CCDRole role : association.getRoles()) {
				rolesContainer.widgets.put(
						role.getId().toString(),
						this.buildWidgetsBlock(role));
			}
			
			attributesContainer.label = ATTRIBUTES_LABEL;
			for (CCDAttribute attribute :
				this.getAttributesFromPropertiesSet(
						association.getProperties())) {
				attributesContainer.widgets.put(
						attribute.getId().toString(),
						this.buildWidgetsBlock(attribute));
			}
			
			operationsContainer.label = OPERATIONS_LABEL;
			for (CCDOperation operation :
				this.getOperationsFromPropertiesSet(
						association.getProperties())) {
				operationsContainer.widgets.put(
						operation.getId().toString(),
						this.buildWidgetsBlock(operation));
			}

			container.widgets.put(
					NAME_IDENTIFIER, associationNameTextField);
			container.widgets.put(
					ATTRIBUTES_IDENTIFIER, attributesContainer);
			container.widgets.put(
					OPERATIONS_IDENTIFIER, operationsContainer);
		}
		return container;
	}
	
	private List<CCDAttribute> getAttributesFromPropertiesSet(
			SortedSet<CCDProperty> propertiesSet) {
		List<CCDAttribute> attributesSet = new ArrayList<CCDAttribute>();
		for (CCDProperty property : propertiesSet) {
			if (property instanceof CCDAttribute) {
				attributesSet.add((CCDAttribute)property);
			}
		}
		return attributesSet;
	}
	
	private List<CCDOperation> getOperationsFromPropertiesSet(
			SortedSet<CCDProperty> propertiesSet) {
		List<CCDOperation> operationsSet = new ArrayList<CCDOperation>();
		for (CCDProperty property : propertiesSet) {
			if (property instanceof CCDOperation) {
				operationsSet.add((CCDOperation)property);
			}
		}
		return operationsSet;
	}

	public void widgetValueChanged(WidgetEvent wEvent) {
		// TODO Auto-generated method stub		
	}
}