package it.uniroma1.dis.mfis.redia.verifint.control.provers;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import it.uniroma1.dis.mfis.redia.verifint.control.ExecutionProperties;
import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProverProgramOutput;
import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProvingResult;
import it.uniroma1.dis.mfis.redia.verifint.control._exceptions.ApplicationException;
import it.uniroma1.dis.mfis.redia.verifint.model.Diagram;

public abstract class Prover {
	public static final String EMPTY_PROVER_PROGRAM_PATH = "";
	public static final String EMPTY_COMMAND_STRING = "";
	public static final String EMPTY_XSLT_PATH = "";
	public static final String EXECUTION_PROPERTIES_NOT_SET_EXCEPTION_MESSAGE
		= "All execution properties must be set before invoking any method";

	protected Diagram diagram;
	private String proverProgramPath;
	private String commandString;
	private String xsltPath;
	
	protected Prover(Diagram diagram) {
		this.diagram = diagram;
		this.proverProgramPath = EMPTY_PROVER_PROGRAM_PATH;
		this.commandString = EMPTY_COMMAND_STRING;
		this.xsltPath = EMPTY_XSLT_PATH;
	}

	protected Diagram getDiagram() {
		return diagram;
	}

	protected void setDiagram(Diagram diagram) {
		this.diagram = diagram;
	}

	public String getProverProgramPath() {
		return proverProgramPath;
	}

	private boolean setProverProgramPath(String proverProgramPath) {
		if (this.proverProgramPath.equals(EMPTY_PROVER_PROGRAM_PATH))
			this.proverProgramPath = proverProgramPath;
		else
			return false;
		return true;
	}

	public String getCommandString() {
		return commandString;
	}

	private boolean setCommandString(String commandString) {
		if (this.commandString.equals(EMPTY_COMMAND_STRING))
			this.commandString = commandString;
		else
			return false;
		return true;
	}

	public String getXsltPath() {
		return xsltPath;
	}

	private boolean setXsltPath(String xsltPath) {
		if (this.xsltPath.equals(EMPTY_XSLT_PATH))
			this.xsltPath = xsltPath;
		else
			return false;
		return true;
	}

	protected boolean setExecutionProperties(
			String proverProgramPathKey,
			String commandStringKey,
			String xsltPathKey) {
		ExecutionProperties properties =
			ExecutionProperties.getInstance();
		return
			this.setProverProgramPath(
					properties.getProperty(proverProgramPathKey)
				) &&
			this.setCommandString(
					properties.getProperty(commandStringKey)
				) &&
			this.setXsltPath(
					properties.getProperty(xsltPathKey)
				);
	}

	public abstract ProvingResult prove(String goal);
	
	protected String createProverProgramInput(String diagramXml) {
		String proverInput = "";
		try {
			// read the XML input
			Source xmlSource = new StreamSource(new StringReader(diagramXml));
			// read the XSLT sheet
			Source xsltSource = new StreamSource(
					new FileReader(
							System.getProperty("user.dir") + "/" +
							this.getXsltPath()
					)
			);
			// transform the XML following the XSLT templates
			TransformerFactory transFactory = TransformerFactory.newInstance();
			Transformer trans = transFactory.newTransformer(xsltSource);
			StringWriter newXmlWriter = new StringWriter();
			trans.transform(xmlSource, new StreamResult(newXmlWriter));
			
			// delete indentations
			String[] lines = newXmlWriter.toString().split("\\n");
			StringBuffer stringBuffer = new StringBuffer();
			for(String line : lines) {
				line = line.trim();
				if (!line.equals("")) {
					stringBuffer.append(line + "\n");
				}
			}
			proverInput = stringBuffer.toString();
		}
		catch (Exception e) {
			throw new ApplicationException(e.getMessage());
		}
		return proverInput;
	}
	
	protected ProverProgramOutput proveByProgram(String proverInput) {
		int exitStatus = 0;
		String outputText = "";
		String errorText = "";
		try {
			
			Process process = Runtime.getRuntime().exec(
					buildProcessCommand(
							this.getCommandString(),
							this.getProverProgramPath()));
			/*
			 * Java 1.5 does not change the working directory, even though you 
			 * specified it; so, the following clean lines of code are useless,
			 * at least on Linux
			
			Process process = Runtime.getRuntime().exec(
					this.getCommandString(),
					new String[0],
					new File(this.getProverProgramPath()));
			 */
			
			PrintWriter inputWriter = new PrintWriter(
					process.getOutputStream());

			inputWriter.println(proverInput);
			inputWriter.flush();
			inputWriter.close();

			BufferedReader outputBufferedReader = new BufferedReader(
					new InputStreamReader(process.getInputStream()));
			String outputLine = outputBufferedReader.readLine();
			while (outputLine != null) {
				outputText += "\n" + outputLine;
				outputLine = outputBufferedReader.readLine();
			}

			BufferedReader errorBufferedReader = new BufferedReader(
					new InputStreamReader(process.getErrorStream()));
			String errorLine = errorBufferedReader.readLine();
			while (errorLine != null) {
				errorText += "\n" + errorLine;
				errorLine = outputBufferedReader.readLine();
			}			
			
			outputBufferedReader.close();
			errorBufferedReader.close();
			exitStatus = process.waitFor();
		}
		catch (Exception e) {
			System.err.println(e);
		}
		return new ProverProgramOutput(exitStatus, outputText, errorText);
	}
	
	protected static String[] buildProcessCommand(
			String commandLine, String workingDirectoryPath) {
		String[] commandLineTokens = commandLine.split("\\s");
		commandLineTokens[0] = workingDirectoryPath + commandLineTokens[0];
		return commandLineTokens;
	}
}