package it.uniroma1.dis.mfis.redia.verifint.control.provers.analysis.classDiagram;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.io.PrintWriter;

import it.uniroma1.dis.mfis.redia.verifint.control.ExecutionProperties;
import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProverProgramOutput;
import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProvingResult;
import it.uniroma1.dis.mfis.redia.verifint.control._exceptions.ApplicationException;
import it.uniroma1.dis.mfis.redia.verifint.control.io.XmlMarshaller;
import it.uniroma1.dis.mfis.redia.verifint.control.provers.Prover;
import it.uniroma1.dis.mfis.redia.verifint.control.provers._constants.Prover9Constants;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDDiagram;

public class CCDProver9Prover extends Prover {
	private String outputElaboratorCommand;

	public CCDProver9Prover(CCDDiagram diagram) {
		super(diagram);
		super.setExecutionProperties(
				Prover9Constants.PROVER_9_PROGRAM_PATH_KEY,
				Prover9Constants.PROVER_9_COMMAND_STRING_KEY,
				Prover9Constants.PROVER_9_XSLT_PATH_KEY);
		this.setOutputElaboratorCommand(
			ExecutionProperties.getInstance().getProperty(
					Prover9Constants.PROOFTRANS_COMMAND_STRING_KEY)
		);
	}

	private String getOutputElaboratorCommand() {
		return outputElaboratorCommand;
	}

	private void setOutputElaboratorCommand(String outputElaboratorCommand) {
		this.outputElaboratorCommand = outputElaboratorCommand;
	}

	@Override
	public ProvingResult prove(String goal) {
		if (this.getCommandString().equals(EMPTY_COMMAND_STRING)
				||
				this.getProverProgramPath().equals(EMPTY_PROVER_PROGRAM_PATH)
				||
				this.getXsltPath().equals(EMPTY_XSLT_PATH)) {
			throw new IllegalStateException(
					EXECUTION_PROPERTIES_NOT_SET_EXCEPTION_MESSAGE);
		}
		
		String diagramXml =
			new XmlMarshaller().marshal((CCDDiagram)this.diagram);
		
		String proverInput =
			Prover9Constants.PROVER_9_ASSUMPTION_STRING +
			super.createProverProgramInput(diagramXml) +
			Prover9Constants.PROVER_9_END_OF_ASSUMPTION_STRING +
			Prover9Constants.PROVER_9_GOALS_STRING +
			goal + "\n" +
			Prover9Constants.PROVER_9_END_OF_GOALS_STRING;

		ProverProgramOutput proverProgramOutput =
			super.proveByProgram(proverInput);
		
		ProvingResult tmpProvingResult =
			new ProvingResult(
					(proverProgramOutput.exitStatus == 0),
					proverProgramOutput.output); 
		
		if (this.getOutputElaboratorCommand() == null) {
			return tmpProvingResult;
		}
		
		return new ProvingResult(
				tmpProvingResult.proven,
				this.extractProof(proverProgramOutput)
			);
	}
	
	protected String extractProof(ProverProgramOutput proverProgramOutput) {
		String proofText = "";
		try {
			Process process = Runtime.getRuntime().exec(
					buildProcessCommand(
							this.outputElaboratorCommand,
							super.getProverProgramPath()));
			/*
			 * Java 1.5 does not change the working directory, even though you 
			 * specified it; so, the following clean lines of code are useless,
			 * at least on Linux
			
			Process process = Runtime.getRuntime().exec(
					this.outputElaboratorCommand,
					new String[0],
					new File(super.getProverProgramPath()));
			 */
			
			PrintWriter writer = new PrintWriter(
					process.getOutputStream());
			String proofTransInput = proverProgramOutput.output;
			writer.println(proofTransInput);
			writer.flush();
			writer.close();

			BufferedReader bufferedReader = new BufferedReader(
					new InputStreamReader(process.getInputStream()));
			String proofLine = bufferedReader.readLine();
			while (proofLine != null) {
				proofText += "\n" + proofLine;
				proofLine = bufferedReader.readLine();
			}
			
			bufferedReader.close();
		}
		catch (Exception e) {
			throw new ApplicationException(e.getMessage());
		}
		return proofText;
	}
}