package it.uniroma1.dis.mfis.redia.verifint.model._dataTypes;

import javax.xml.bind.annotation.XmlAttribute;

public class Multiplicity {
	public static final Short UNBOUNDED = Short.MAX_VALUE;

	public static final String INCORRECT_MINIMUM_MULTIPLICITY_EXCEPTION =
		"Minimum multiplicity must be an integer value equal or greater than 0";

	public static final String
		INCORRECT_MAXIMUM_MULTIPLICITY_EXCEPTION =
		"Maximum multiplicity must be an integer value equal or greater than 1";
	
	private static final String MIN_MAX_SEPARATOR = " .. ";

	@XmlAttribute(required=true)
	private Short minimum;
	@XmlAttribute(required=true)
	private Short maximum;	
	
	protected Multiplicity() {
		super();
	}
	
	public Multiplicity(Short minimum, Short maximum) {
		super();
		this.setMinimum(minimum);
		this.setMaximum(maximum);
	}

	public Short getMinimum() {
		return minimum;
	}
	
	private void setMinimum(Short minimum) {
		if (minimum >= 0)
			this.minimum = minimum;
		else
			throw new IllegalArgumentException(
					INCORRECT_MINIMUM_MULTIPLICITY_EXCEPTION
				);
	}
	
	public Short getMaximum() {
		return maximum;
	}
	
	private void setMaximum(Short maximum) {
		if (maximum >= 1)
			this.maximum = maximum;
		else
			throw new IllegalArgumentException(
					INCORRECT_MAXIMUM_MULTIPLICITY_EXCEPTION
				);
	}

	@Override
	public boolean equals(Object o) {
		if (!(o instanceof Multiplicity))
			return false;
		Multiplicity m = (Multiplicity)o;
		return
			this.getMinimum().equals(m.getMinimum())
			&&
			this.getMaximum().equals(m.getMaximum());
	}

	@Override
	public String toString() {
		String s = this.minimum.toString();
		s += MIN_MAX_SEPARATOR;
		s += (this.maximum.equals(UNBOUNDED) ? "*" : this.maximum);
		return s;
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		return super.clone();
	}
}