package it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.xml.bind.annotation.*;

import it.uniroma1.dis.mfis.redia.verifint.model._exceptions.AssociationTargetException;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.NameIdentifiable;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams._links.CCDAssociation_hasRoles_CCDRole;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams._links.CCDAssociation_isSubSetOf_CCDSpecialization;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams._links.CCDAssociation_isSuperSetOf_CCDSpecialization;

public class CCDAssociation extends CCDRelation {
	private static final long serialVersionUID = -4031982799441584589L;
	public static final String DEFAULT_IDENTIFIER_NAME_INFIX = "_associates_";
	
	/**
	 * The list of roles involved.
	 * <br />Beware the list order counts: that is, if class <em>B</em> is the first in this list, and <em>A</em> is the second, this association will be intended as a link that goes FROM <em>B</em> TO <em>A</em>.
	 */
	@XmlElementWrapper(name="roles")
	@XmlElement(name="role")
	private List<CCDRole> roles;
	@XmlTransient
	private SortedSet<CCDSpecialization> specializationsWhoseSubSet;
	@XmlTransient
	private SortedSet<CCDSpecialization> specializationsWhoseSuperSet;

	protected CCDAssociation() {
		super();
		this.roles = new ArrayList<CCDRole>();
		this.specializationsWhoseSubSet = new TreeSet<CCDSpecialization>();
		this.specializationsWhoseSuperSet = new TreeSet<CCDSpecialization>();
	}
	
	CCDAssociation(String name) {
		super(name);
		this.roles = new ArrayList<CCDRole>();
		this.specializationsWhoseSubSet = new TreeSet<CCDSpecialization>();
		this.specializationsWhoseSuperSet = new TreeSet<CCDSpecialization>();
	}

	public List<CCDRole> getRoles() {
		return this.roles;
	}
	
	protected void setRoles(List<CCDRole> roles) {
		this.roles = roles;
	}

	public SortedSet<CCDSpecialization> getSpecializationsWhoseSubSet() {
		return
			new TreeSet<CCDSpecialization>(this.specializationsWhoseSubSet);
	}

	protected void setSpecializationsWhoseSubSet(
			SortedSet<CCDSpecialization> specializationWhoseSubSet) {
		this.specializationsWhoseSubSet = specializationWhoseSubSet;
		super.dependenciesWhoseSource.addAll(this.specializationsWhoseSubSet);
	}

	public SortedSet<CCDSpecialization> getSpecializationsWhoseSuperSet() {
		return 
			new TreeSet<CCDSpecialization>(this.specializationsWhoseSuperSet);
	}

	protected void setSpecializationsWhoseSuperSet(
			SortedSet<CCDSpecialization> specializationWhoseSuperSet) {
		this.specializationsWhoseSuperSet = specializationWhoseSuperSet;
		super.dependenciesWhoseDestination.addAll(this.specializationsWhoseSuperSet);
	}

	public void setRoles(CCDAssociation_hasRoles_CCDRole link)
	throws AssociationTargetException {
		if (!link.getAssociation().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.setRoles(link.getRoles());
	}

	public void removeRole(
			CCDAssociation_hasRoles_CCDRole link)
	throws AssociationTargetException {
		if (!link.getAssociation().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.roles.remove(link.getRoles().iterator().next());
	}

	public void addSpecializationWhoseSubSet(
			CCDAssociation_isSubSetOf_CCDSpecialization link)
	throws AssociationTargetException {
		if (!link.getAssociation().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.specializationsWhoseSubSet.add(
				link.getSpecialization());
		super.addDependencyWhoseSource(link.getSpecialization());
	}

	public void removeSpecializationWhoseSubSet(
			CCDAssociation_isSubSetOf_CCDSpecialization link)
	throws AssociationTargetException {
		if (!link.getAssociation().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.specializationsWhoseSubSet.remove(
				link.getSpecialization());
		super.removeDependencyWhoseSource(link.getSpecialization());
	}

	public void addSpecializationWhoseSuperSet(
			CCDAssociation_isSuperSetOf_CCDSpecialization link)
	throws AssociationTargetException {
		if (!link.getAssociation().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.specializationsWhoseSuperSet.add(
				link.getSpecialization());
		super.addDependencyWhoseDestination(link.getSpecialization());
	}

	public void removeSpecializationWhoseSuperSet(
			CCDAssociation_isSuperSetOf_CCDSpecialization link)
	throws AssociationTargetException {
		if (!link.getAssociation().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.specializationsWhoseSuperSet.remove(
				link.getSpecialization());
		super.removeDependencyWhoseDestination(link.getSpecialization());
	}	

	@Override
	public String getIdentifierName() {
		String identifierName =
			super.getIdentifierName() +
			NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
			DEFAULT_IDENTIFIER_NAME_INFIX;
		for (CCDRole role : this.getRoles()) {
			identifierName +=
				NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
				role.getParticipantClass().getName() +
				NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
				role.getName();
		}
		return identifierName;
	}
}