package it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams;

import java.util.ArrayList;
import java.util.List;

import it.uniroma1.dis.mfis.redia.verifint.model._dataTypes.Multiplicity;
import it.uniroma1.dis.mfis.redia.verifint.model._exceptions.MultiplicityException;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams._links.CCDAssociation_hasRoles_CCDRole;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams._links.CCDClass_participates_CCDRole;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams._links.CCDDiagram_isComposedBy_CCDElement;

public class CCDAssociation_Factory {
	public CCDAssociation association;
	public List<CCDRole> roles;

	public Association_Roles_Container createInstances(
			CCDDiagram diagram,
			String associationName,
			List<Role_TemporaryContainer> hatred_roles 
	)
	throws MultiplicityException
	{
		this.association = new CCDAssociation(associationName);
		this.roles = new ArrayList<CCDRole>();
		
		for (Role_TemporaryContainer role : hatred_roles) {
			this.roles.add(role.createInstance());
		}
		
		this.resolveLinkConstraints(diagram);
		return
			new Association_Roles_Container(this.association, this.roles);
	}

	private void resolveLinkConstraints(CCDDiagram diagram)
	throws MultiplicityException
	{
		CCDAssociation_hasRoles_CCDRole.addLinks(
				this.association, this.roles);
		CCDDiagram_isComposedBy_CCDElement.addLink(
				diagram, this.association);
		for (CCDRole role : this.roles) {
			CCDDiagram_isComposedBy_CCDElement.addLink(diagram, role);
		}
	}

	public static class Role_TemporaryContainer {
		public final String name;
		public final Multiplicity multiplicity;
		public final CCDClass participantClass;

		public Role_TemporaryContainer(String name,
				Multiplicity multiplicity, CCDClass participantClass) {
			this.name = name;
			this.multiplicity = multiplicity;
			this.participantClass = participantClass;
		}

		public Role_TemporaryContainer(
				Multiplicity multiplicity, CCDClass participantClass) {
			this(null, multiplicity, participantClass);
		}

		public Role_TemporaryContainer(String name, CCDClass participantClass) {
			this(name, null, participantClass);
		}

		public Role_TemporaryContainer(CCDClass participantClass) {
			this(null, null, participantClass);
		}
		
		private void resolveLinkConstraints(CCDRole role, CCDClass participantClass) {
			CCDClass_participates_CCDRole.addLink(
					role, participantClass);
		}

		private CCDRole createInstance() {
			CCDRole role = null;
			if (this.multiplicity == null) {
				if (this.name == null) {
					role = new CCDRole();
				}
				else {
					role = new CCDRole(this.name);
				}
			} else {
				if (this.name == null) {
					role = new CCDRole(this.multiplicity);
				}
				else {
					role = new CCDRole(this.name, this.multiplicity);
				}
			}
			this.resolveLinkConstraints(role, this.participantClass);
			return role;
		}
	}

	public class Association_Roles_Container {
		public CCDAssociation association;
		public List<CCDRole> roles;

		private Association_Roles_Container(CCDAssociation association,
				List<CCDRole> roles) {
			this.association = association;
			this.roles = roles;
		}
	}
}