package it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams;

import java.util.TreeSet;
import java.util.SortedSet;

import javax.xml.bind.annotation.*;

import it.uniroma1.dis.mfis.redia.verifint.model._exceptions.AssociationTargetException;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.NameIdentifiable;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams._links.CCDClass_isBaseClassOf_CCDGeneralization;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams._links.CCDClass_isSubClassOf_CCDGeneralization;

public class CCDGeneralization extends CCDComponent {
	private static final long serialVersionUID = 617375476355102160L;
	public static final String DEFAULT_IDENTIFIER_NAME_INFIX = "_generalizes_";
	
	@XmlElementWrapper(name="subClasses")
	@XmlElement(name="subClass")
	@XmlIDREF
	private SortedSet<CCDClass> subClasses;
	@XmlElement
	@XmlIDREF
	private CCDClass baseClass;
	@XmlAttribute(required=false)
	private Boolean disjoint;
	@XmlAttribute(required=false)
	private Boolean complete;

	CCDGeneralization(Boolean disjoint, Boolean complete) {
		super();
		this.disjoint = disjoint;
		this.complete = complete;
		this.subClasses = new TreeSet<CCDClass>();
	}

	CCDGeneralization(String name, Boolean disjoint, Boolean complete) {
		super(name);
		this.disjoint = disjoint;
		this.complete = complete;
		this.subClasses = new TreeSet<CCDClass>();
	}

	CCDGeneralization(String name) {
		this(name, false, false);
	}

	CCDGeneralization() {
		this(false, false);
	}

	public boolean isA() {
		return this.subClasses.size() == 1;
	}

	public Boolean isDisjoint() {
		return this.disjoint;
	}

	public void setDisjoint(Boolean disjoint) {
		this.disjoint = disjoint;
	}

	public Boolean isComplete() {
		return this.complete;
	}

	public void setComplete(Boolean complete) {
		this.complete = complete;
	}

	public SortedSet<CCDClass> getSubClasses() {
		return new TreeSet<CCDClass>(this.subClasses);
	}

	protected void setSubClasses(SortedSet<CCDClass> subClasses) {
		this.subClasses = subClasses;
	}

	public CCDClass getBaseClass() {
		return this.baseClass;
	}

	protected void setBaseClass(CCDClass baseClass) {
		this.baseClass = baseClass;
	}

	public void setBaseClass(CCDClass_isBaseClassOf_CCDGeneralization link)
	throws AssociationTargetException {
		if (!link.getGeneralization().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.setBaseClass(link.getBaseClass());
	}

	public void unsetBaseClass(CCDClass_isBaseClassOf_CCDGeneralization link)
	throws AssociationTargetException {
		if (!link.getGeneralization().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.baseClass = null;
	}

	public void setSubClass(
			CCDClass_isSubClassOf_CCDGeneralization link)
	throws AssociationTargetException {
		if (!link.getGeneralization().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.subClasses = new TreeSet<CCDClass>();
		this.addSubClass(link);
	}

	public void addSubClass(
			CCDClass_isSubClassOf_CCDGeneralization link)
	throws AssociationTargetException {
		if (!link.getGeneralization().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.subClasses.add(link.getSubClass());
	}

	public void removeSubClass(
			CCDClass_isSubClassOf_CCDGeneralization link)
	throws AssociationTargetException {
		if (!link.getGeneralization().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.subClasses.remove(link.getSubClass());
	}

	@Override
	public String getIdentifierName() {
		String identifierName =
			super.getIdentifierName() +
			NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
			this.getBaseClass().getName() +
			NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
			DEFAULT_IDENTIFIER_NAME_INFIX;
		for (CCDClass subClass : this.getSubClasses()) {
			identifierName +=
				NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
				subClass.getName();
		}
		return identifierName;
	}
}