package it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.*;

import it.uniroma1.dis.mfis.redia.verifint.model._dataTypes.Multiplicity;
import it.uniroma1.dis.mfis.redia.verifint.model._exceptions.AssociationTargetException;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.NameIdentifiable;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams._links.CCDOperation_hasParameter_CCDParameter;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.operations._interfaces.COpOperation;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.operations._interfaces.COpParameter;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types._interfaces.CTType;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types._links.CTType_isTypeOf_CTTypeDefined;

public class CCDOperation extends CCDProperty implements COpOperation {
	private static final long serialVersionUID = 3195913016547031502L;
	private static final String VOID = "void";

	@XmlElementWrapper(name="inputParameters")
	@XmlElement(name="parameter", type=CCDParameter.class)
	private List<CCDParameter> inputParameters;

	CCDOperation(String name, Multiplicity multiplicity) {
		super(name, multiplicity);
		this.inputParameters = new ArrayList<CCDParameter>();
	}

	CCDOperation(String name) {
		this(name,
				new Multiplicity(
						DEFAULT_MINIMUM_MULTIPLICITY,
						DEFAULT_MAXIMUM_MULTIPLICITY
					)
				);
	}
	
	protected CCDOperation() {
		super();
	}

	public List<COpParameter> getInputParameters() {
		return new ArrayList<COpParameter>(this.inputParameters);
	}

	protected void setInputParameters(List<CCDParameter> inputParameters) {
		this.inputParameters = inputParameters;
	}

	public void addInputParameter(CCDOperation_hasParameter_CCDParameter link)
	throws AssociationTargetException {
		if (!link.getOperation().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.inputParameters.add(link.getParameter());
	}

	public void removeInputParameter(CCDOperation_hasParameter_CCDParameter link)
	throws AssociationTargetException {
		if (!link.getOperation().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.inputParameters.remove(link.getParameter());
	}

	/**
	 * <br />Behaves like
	 * 	{@link it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDProperty#getType() CCDProperty.getType()}
	 * @see it.uniroma1.dis.mfis.redia.verifint.model.analysis.operations._interfaces.COpOperation#getReturnType()
	 */
	public CTType getReturnType() {
		return super.getType();
	}

	protected void setReturnType(CTType returnType) {
		super.setType(returnType);
	}
	
	@XmlElement
	private String getReturnTypeString() {
		return (this.getReturnType() == null) ?
			CCDOperation.VOID :
				this.getReturnType().getName();
	}

	public void setType(CTType_isTypeOf_CTTypeDefined link)
	throws AssociationTargetException {
		if (!link.getTypeDefined().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.setReturnType(link.getType());
	}
	
	/**
	 * An <em>alias</em> of
	 * 	{@link #setType(CTType_isTypeOf_CTTypeDefined)
	 * 		this.setType(CTType_isTypeOf_CTTypeDefined)} 
	 * @param link
	 * @throws AssociationTargetException
	 */
	public void setReturnType(CTType_isTypeOf_CTTypeDefined link)
	throws AssociationTargetException {
		this.setType(link);
	}

	@Override
	public String getIdentifierName() {
		String identifierName =
			this.getOwnerRelation().getIdentifierName() +
			NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
			this.getName();
		for (CCDParameter param : this.inputParameters) {
			identifierName +=
				NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
				param.getName() +
				NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
				param.getType().getName();
		}
		return identifierName +
			NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
			this.getReturnTypeString();
	}
}