package it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams._links;

import java.util.ArrayList;
import java.util.List;

import it.uniroma1.dis.mfis.redia.verifint.model._exceptions.MultiplicityException;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDAssociation;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDRole;


public class CCDAssociation_hasRoles_CCDRole {
	public static final Integer MINIMUM_NUMBER_OF_ROLES_INVOLVED = 2;

	private static final String
	MINIMUM_NUMBER_OF_ROLES_INVOLVED_EXCEPTION_MESSAGE =
		"The number of roles for an association must be " +
		"equal or greater than " +
		MINIMUM_NUMBER_OF_ROLES_INVOLVED.toString();

	private List<CCDRole> roles;
	private CCDAssociation association;

	private CCDAssociation_hasRoles_CCDRole(List<CCDRole> roles,
			CCDAssociation association)
	throws MultiplicityException {
		if (roles.size() < MINIMUM_NUMBER_OF_ROLES_INVOLVED)
			throw new MultiplicityException(
				MINIMUM_NUMBER_OF_ROLES_INVOLVED_EXCEPTION_MESSAGE);
		this.roles = roles;
		this.association = association;
	}

	/**
	 * Builds <code>CCDAssociation_hasRoles_CCDRole</code> instances.
	 * @param roles
	 * @param association
	 */
	private CCDAssociation_hasRoles_CCDRole(CCDRole role,
			CCDAssociation association) {
		ArrayList<CCDRole> roles = new ArrayList<CCDRole>();
		roles.add(role);
		this.roles = roles;
		this.association = association;
	}

	public List<CCDRole> getRoles() {
		return this.roles;
	}

	public CCDAssociation getAssociation() {
		return this.association;
	}

	/**
	 * @param association
	 * @param roles
	 * @throws MultiplicityException when specified in
	 * 	{@link it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDAssociation#setRoles(CCDAssociation_hasRoles_CCDRole) CCDAssociation.setRoles(CCDAssociation_hasRoles_CCDRole)}
	 */
	public static void addLinks(
			CCDAssociation association,
			List<CCDRole> roles)
	throws MultiplicityException {
		CCDAssociation_hasRoles_CCDRole self =
			new CCDAssociation_hasRoles_CCDRole(roles, association);
		for (CCDRole role : roles) {
			role.setAssociation(self);
		}
		association.setRoles(self);
	}

	/**
	 * <br/>It doesn't check the number of roles remaining for the specified association, because it's allowed for an association to have a momentary illegal state - e.g. while substituting a roles with another  
	 * @param association
	 * @param roles
	 */
	public static void removeLink(
			CCDAssociation association,
			CCDRole role) {
		CCDAssociation_hasRoles_CCDRole self =
			new CCDAssociation_hasRoles_CCDRole(role, association);
		role.unsetAssociation(self);
		association.removeRole(self);
	}
}