package it.uniroma1.dis.mfis.redia.verifint.model.analysis.types;

import java.util.SortedSet;
import java.util.TreeSet;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.XmlTransient;

import it.uniroma1.dis.mfis.redia.verifint.model._dataTypes.ElementId;
import it.uniroma1.dis.mfis.redia.verifint.model._exceptions.AssociationTargetException;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.Identifiable;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.NameIdentifiable;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types._interfaces.CTDataTypeDefined;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types._interfaces.CTType;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types._interfaces.CTTypeDefined;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types._links.CTDataType_isTypeOf_CTDataTypeDefined;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types._links.CTType_isTypeOf_CTTypeDefined;
import it.uniroma1.dis.mfis.redia.verifint.model.constraints._interfaces.XConstrainable;
import it.uniroma1.dis.mfis.redia.verifint.model.constraints._interfaces.XConstraint;
import it.uniroma1.dis.mfis.redia.verifint.model.constraints._links.XConstrainable_satisfies_XConstraint;

public abstract class CTDataType
implements CTType, NameIdentifiable, XConstrainable {
	public static Long id_assigner = 1L;
	public static final String ELEMENT_PREFIX = "CT";
	public static final String ID_PREFIX = "_Type_" + ELEMENT_PREFIX;
	
	@XmlID
	private String name;
	@XmlAttribute
	private ElementId id;
	@XmlTransient
	private SortedSet<CTTypeDefined> definedElements;
	@XmlTransient
	private SortedSet<CTDataTypeDefined> definedDataElements;
	@XmlTransient
	private SortedSet<XConstraint> constraints;

	public CTDataType(String name) {
		synchronized (id_assigner) {
			this.setId(CTDataType.id_assigner++);
		}
		this.name = name;
		this.definedElements = new TreeSet<CTTypeDefined>();
		this.definedDataElements = new TreeSet<CTDataTypeDefined>();
	}

	protected CTDataType() {
		super();
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public SortedSet<CTTypeDefined> getDefinedElements() {
		return new TreeSet<CTTypeDefined>(definedElements);
	}

	protected void setDefinedElements(
			SortedSet<CTTypeDefined> definedElements) {
		this.definedElements = definedElements;
	}

	public SortedSet<CTDataTypeDefined> getDefinedDataElements() {
		return new TreeSet<CTDataTypeDefined>(definedDataElements);
	}

	protected void setDefinedDataElements(
			SortedSet<CTDataTypeDefined> definedDataElements) {
		this.definedDataElements = definedDataElements;
	}

	public void addTypeDefinedElement(CTType_isTypeOf_CTTypeDefined link)
	throws AssociationTargetException {
		if (!link.getType().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.definedElements.add(link.getTypeDefined());
	}

	public void removeTypeDefinedElement(CTType_isTypeOf_CTTypeDefined link)
	throws AssociationTargetException {
		if (!link.getType().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.definedElements.remove(link.getTypeDefined());
	}

	public void addDataTypeDefinedElement(
			CTDataType_isTypeOf_CTDataTypeDefined link) {
		if (!link.getDataType().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.definedDataElements.add(link.getDataTypeDefined());
	}

	public void removeDataTypeDefinedElement(
			CTDataType_isTypeOf_CTDataTypeDefined link) {
		this.definedDataElements.add(link.getDataTypeDefined());
		
	}

	public SortedSet<XConstraint> getConstraints() {
		return constraints;
	}

	public void setConstraints(SortedSet<XConstraint> constraints) {
		this.constraints = constraints;
	}
	
	public void addConstraint(XConstrainable_satisfies_XConstraint link)
	throws AssociationTargetException {
		if (!link.getConstrainedElement().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.constraints.add(link.getConstraint());
	}

	public void removeConstraint(XConstrainable_satisfies_XConstraint link)
	throws AssociationTargetException {
		if (!link.getConstrainedElement().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.constraints.remove(link.getConstraint());
	}

	public ElementId getId() {
		return this.id;
	}

	private void setId(Long numericSuffix) {
		this.setId(new ElementId(ID_PREFIX + numericSuffix.toString()));
	}

	protected void setId(ElementId id) {
		this.id = id;
	}

	@Override
	public String toString() {
		return this.getIdentifierName();
	}

	public String getIdentifierName() {
		return this.getName();
	}

	public int compareTo(Identifiable o) {
		return
			this.getId().compareTo(o.getId());
	}
}