package it.uniroma1.dis.mfis.redia.verifint.test;

import it.uniroma1.dis.mfis.redia.verifint.test.testers.DiagramToXmlTester;
import it.uniroma1.dis.mfis.redia.verifint.test.testers.GfxTester;
import it.uniroma1.dis.mfis.redia.verifint.test.testers.PropertyProvingTester;
import it.uniroma1.dis.mfis.redia.verifint.test.testers.XslTester;

public class MainReDiAVeriFIntTest {
	public static final String ANIMALS_TEST_CODE = "animals";
	public static final String BANK_TRANSFERS_TEST_CODE = "bank";
	public static final String ENGLISH_ITALIANS_TEST_CODE = "engita";
	public static final String HOSPITAL_TEST_CODE = "hospital";
	public static final String SOLDIERS_TEST_CODE = "soldiers";
	public static final String STUDENT_TEST_CODE = "student";
	
	public static final String GFX_TEST_COMMAND = "gfx";
	public static final String XSLT_TEST_COMMAND = "xslt";
	public static final String XML_TEST_COMMAND = "xml";
	public static final String PROVER_TEST_COMMAND = "prover";
	
	public static final String SHELL_COMMAND_NAME = "rediaverifint";
	
	public static final String TEST_PREFIX_ARGUMENT_COMMAND = "-test";

	public static final String TEST_NOT_AVAILABLE_MESSAGE =
		"The required test is not available";
	public static final String DEFAULT_TEST_CODE = "";
	public static final String TRY_MORE_MESSAGE = "\n" +
		"Try something more now!" + "\n";
	
	public static void main(String[] args) {		
		if (args.length > 1 && !args[0].equals(TEST_PREFIX_ARGUMENT_COMMAND)) {
			System.err.println(writeUsageInstructions());
			System.exit(1);
		}
		switch (args.length) {
			case 0: case 1:
				new PropertyProvingTester().test(
						MainReDiAVeriFIntTest.ENGLISH_ITALIANS_TEST_CODE);
				System.out.println(TRY_MORE_MESSAGE);
				System.out.println(writeUsageInstructions());
				break;
			case 3:
				if (args[1].equals(GFX_TEST_COMMAND)) {
					if (!(new GfxTester().test(args[2]))) {
						System.err.println(TEST_NOT_AVAILABLE_MESSAGE);
						System.err.println(writeUsageInstructions());
					}
				}
				else if (args[1].equals(XSLT_TEST_COMMAND)) {
					try {
						if (!(new XslTester().test(args[2]))) {
							System.err.println(TEST_NOT_AVAILABLE_MESSAGE);
							System.err.println(writeUsageInstructions());
						}
					} catch (Exception e) {
						System.err.println(e.getMessage());
						System.exit(1);
					}
					System.exit(0);
				}
				else if (args[1].equals(XML_TEST_COMMAND)) {
					try {
						if (!(new DiagramToXmlTester().test(args[2]))) {
							System.err.println(TEST_NOT_AVAILABLE_MESSAGE);
							System.err.println(writeUsageInstructions());
						}
					} catch (Exception e) {
						System.err.println(e.getMessage());
						System.exit(1);
					}
					System.exit(0);
				}
				else if (args[1].equals(PROVER_TEST_COMMAND)) {
					if (!new PropertyProvingTester().test(args[2])) {
						System.err.println(TEST_NOT_AVAILABLE_MESSAGE);
						System.err.println(writeUsageInstructions());
					}
					System.exit(0);
				}
				else {
					System.err.println(writeUsageInstructions());
					System.exit(1);
				}
				break;
			case 2:
				if (args[1].equals(GFX_TEST_COMMAND)) {
					new GfxTester().test();
				}
				else if (args[1].equals(XSLT_TEST_COMMAND)) {
					try {
						new XslTester().test();
					} catch (Exception e) {
						System.err.println(e.getMessage());
						System.exit(1);
					}
					System.exit(0);
				}
				else if (args[1].equals(XML_TEST_COMMAND)) {
					try {
						new DiagramToXmlTester().test();
					} catch (Exception e) {
						System.err.println(e.getMessage());
						System.exit(1);
					}
					System.exit(0);
				}
				else if (args[1].equals(PROVER_TEST_COMMAND)) {
					new PropertyProvingTester().test();
					System.exit(0);
				}
				else {
					System.err.println(writeUsageInstructions());
					System.exit(1);
				}
				break;
			default:
				System.err.println(writeUsageInstructions());
				System.exit(1);
		}
	}

	private static String writeUsageInstructions() {
		return 	"Usage: " + SHELL_COMMAND_NAME + " [" + TEST_PREFIX_ARGUMENT_COMMAND + " [TYPE [DIAGRAM]]]" + "\n" +
				"\n" + "TYPEs:" + "\n" +
				"\t" + GFX_TEST_COMMAND + "\t\t(graphic engine test)\n" +
				"\t" + XML_TEST_COMMAND + "\t\t(diagram > xml test)\n" +
				"\t" + XSLT_TEST_COMMAND + "\t\t(diagram > prover-input test)\n" +
				"\t" + PROVER_TEST_COMMAND + "\t\t(diagram > proof)\n" +
				"\n" + "DIAGRAMs:" + "\n" +
				"\t" + ANIMALS_TEST_CODE + "\t\t(works with " + XML_TEST_COMMAND + ", " + XSLT_TEST_COMMAND + " tests)\n" +
				"\t" + BANK_TRANSFERS_TEST_CODE + "\t\t(works with " + XML_TEST_COMMAND + ", " + XSLT_TEST_COMMAND + ", " + GFX_TEST_COMMAND + " tests)\n" +
				"\t" + ENGLISH_ITALIANS_TEST_CODE + "\t\t(works with " + XML_TEST_COMMAND + ", " + XSLT_TEST_COMMAND + ", " + PROVER_TEST_COMMAND + ", " + GFX_TEST_COMMAND + " tests)\n" +
				"\t" + HOSPITAL_TEST_CODE + "\t(works with " + XML_TEST_COMMAND + ", " + XSLT_TEST_COMMAND + ", " + GFX_TEST_COMMAND + " tests)\n" +
				"\t" + SOLDIERS_TEST_CODE + "\t(works with " + XML_TEST_COMMAND + ", " + XSLT_TEST_COMMAND + ", " + GFX_TEST_COMMAND + " tests)\n" +
				"\t" + STUDENT_TEST_CODE + "\t\t(works with " + XML_TEST_COMMAND + ", " + XSLT_TEST_COMMAND + ", " + GFX_TEST_COMMAND + " tests)\n" +
				"\n" + "EXAMPLES of use:" + "\n" +
				"\t" + SHELL_COMMAND_NAME + "\n" +
				"\t" + SHELL_COMMAND_NAME + " " + TEST_PREFIX_ARGUMENT_COMMAND + " " + PROVER_TEST_COMMAND + "\n" +
				"\t" + SHELL_COMMAND_NAME + " " + TEST_PREFIX_ARGUMENT_COMMAND + " " + GFX_TEST_COMMAND + " " + BANK_TRANSFERS_TEST_CODE +
				"\n";
	}
}