package it.uniroma1.dis.mfis.redia.verifint.test.testers;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import it.uniroma1.dis.mfis.redia.verifint.control.io.XmlMarshaller;
import it.uniroma1.dis.mfis.redia.verifint.model._dataTypes.Multiplicity;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.*;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types.*;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types._links.CTType_isTypeOf_CTTypeDefined;
import it.uniroma1.dis.mfis.redia.verifint.test.MainReDiAVeriFIntTest;

public class DiagramToXmlTester {
	private String xmlDiagram = null;

	// Animals diagram
	public CCDDiagram buildAnimalsDiagram() {
		CCDDiagram diagram = new CCDDiagram("AnimalsDiagram");

		CCDClass_Factory class_factory = new CCDClass_Factory();
		CCDOperation_Factory op_factory = new CCDOperation_Factory();
		CCDAttribute_Factory attribute_factory = new CCDAttribute_Factory();
		CCDAssociation_Factory association_factory = new CCDAssociation_Factory();
		CCDParameter_Factory parameter_factory = new CCDParameter_Factory();
		CCDSpecialization_Factory specialization_factory = new CCDSpecialization_Factory();

		// Parent class: Pet
		CCDClass pet_class = class_factory.createInstance(diagram, "Pet");
		// The pet has a name
		CCDAttribute name_attribute = attribute_factory.createInstance(diagram,
				pet_class, CTBaseType.STRING_TYPE, "name");

		// Pet's child class: Dog
		CCDClass dog_class = class_factory.createInstance(diagram, "Dog");
		// The dog can bark
		CCDOperation bark_operation = op_factory.createInstance(diagram,
				"bark", dog_class);

		// The dog can bark at any volume, too!
		CCDOperation bark_volumed_operation = op_factory.createInstance(
				diagram, "bark", dog_class);
		CCDParameter bark_volume_parameter = parameter_factory.createInstance(
				diagram, bark_volumed_operation, CTBaseType.INTEGER_TYPE,
				"volume");
		CTType_isTypeOf_CTTypeDefined.addLink(CTBaseType.STRING_TYPE,
				bark_volumed_operation);

		// Pet's child class: Cat
		CCDClass cat_class = class_factory.createInstance(diagram, "Cat");

		// Pet's child class: Parrot
		CCDClass parrot_class = class_factory.createInstance(diagram, "Parrot");

		// Pet's child class: Rabbit
		CCDClass rabbit_class = class_factory.createInstance(diagram, "Rabbit");

		// Set of child classes
		TreeSet<CCDClass> pet_subClasses = new TreeSet<CCDClass>();
		pet_subClasses.add(dog_class);
		pet_subClasses.add(cat_class);
		pet_subClasses.add(parrot_class);
		pet_subClasses.add(rabbit_class);

		// Doggies and kitties are Pets
		CCDGeneralization pets_generalization = new CCDGeneralization_Factory()
				.createInstance(diagram, true, true, pet_class, pet_subClasses);

		// Dogs don't love cats
		List<CCDAssociation_Factory.Role_TemporaryContainer> hatred_roles =
			new ArrayList<CCDAssociation_Factory.Role_TemporaryContainer>(2);
		// The first in the list is the Dog
		hatred_roles.add(new CCDAssociation_Factory.Role_TemporaryContainer(
				"killer", new Multiplicity((short) 0, (short) 7),
				dog_class));
		// The second in the list is the Cat, thus the association is going to
		// have an implicit direction
		hatred_roles.add(new CCDAssociation_Factory.Role_TemporaryContainer(
				"victim", new Multiplicity((short) 1, Multiplicity.UNBOUNDED),
				cat_class));

		CCDAssociation_Factory.Association_Roles_Container hatred_container =
			association_factory
				.createInstances(diagram, "hates", hatred_roles);

		// Pets argue among them, sometimes
		List<CCDAssociation_Factory.Role_TemporaryContainer> argue_roles =
			new ArrayList<CCDAssociation_Factory.Role_TemporaryContainer>(2);
		// The first in the list is a Pet
		argue_roles.add(new CCDAssociation_Factory.Role_TemporaryContainer(
				"insulted", pet_class));
		// The second in the list is another Pet, thus the association is going
		// to have an implicit direction
		argue_roles.add(new CCDAssociation_Factory.Role_TemporaryContainer(
				"angry", pet_class));

		CCDAssociation_Factory.Association_Roles_Container argue_container =
			association_factory
				.createInstances(diagram, "argues", argue_roles);

		// When a Dog hates a Cat, there's always something to argue (it's a
		// specialization!)
		CCDSpecialization hatred_argue_specialization = specialization_factory
				.createInstance(diagram, argue_container.association,
						hatred_container.association);

		// Another class: Rat (it's not a pet, usually)
		CCDClass rat_class = class_factory.createInstance(diagram, "Rat");
		// Cats hunt rats!
		List<CCDAssociation_Factory.Role_TemporaryContainer> hunt_roles =
			new ArrayList<CCDAssociation_Factory.Role_TemporaryContainer>(
				2);
		hunt_roles.add(new CCDAssociation_Factory.Role_TemporaryContainer(
				"hunter", cat_class));
		hunt_roles.add(new CCDAssociation_Factory.Role_TemporaryContainer(
				new Multiplicity((short)3, Multiplicity.UNBOUNDED), rat_class));

		CCDAssociation_Factory.Association_Roles_Container hunt_container =
			association_factory
				.createInstances(diagram, "hunts", hunt_roles);
		CCDAttribute hunting_speed = attribute_factory.createInstance(diagram,
				hunt_container.association, CTBaseType.FLOAT_TYPE, "speed");

		return diagram;
	}
	
	// English vS Italian diagram
	public CCDDiagram buildEnglishItaliansDiagram() {
		CCDDiagram diagram = new CCDDiagram("EnglishAndItalians");
		
		CCDClass_Factory class_factory = new CCDClass_Factory();
		CCDGeneralization_Factory generalization_factory =
			new CCDGeneralization_Factory();
		
		CCDClass person_class = class_factory.createInstance(
				diagram, "Person");
		CCDClass italian_class = class_factory.createInstance(
				diagram, "Italian");
		CCDClass english_class = class_factory.createInstance(
				diagram, "English");
		CCDClass lazy_class = class_factory.createInstance(
				diagram, "Lazy");
		CCDClass latinLover_class = class_factory.createInstance(
				diagram, "LatinLover");
		CCDClass gentleman_class = class_factory.createInstance(
				diagram, "Gentleman");
		CCDClass hooligan_class = class_factory.createInstance(
				diagram, "Hooligan");

		Set<CCDClass> itaEnBrothers = new TreeSet<CCDClass>();
		itaEnBrothers.add(italian_class);
		itaEnBrothers.add(english_class);
		
		generalization_factory.createInstance(
				diagram, true, false, person_class, itaEnBrothers);

		Set<CCDClass> italianChildrenBrothers = new TreeSet<CCDClass>();
		italianChildrenBrothers.add(lazy_class);
		italianChildrenBrothers.add(latinLover_class);
		
		generalization_factory.createInstance(
				diagram, true, true, italian_class, italianChildrenBrothers);

		Set<CCDClass> englishChildrenBrothers = new TreeSet<CCDClass>();
		englishChildrenBrothers.add(gentleman_class);
		englishChildrenBrothers.add(hooligan_class);
		
		generalization_factory.createInstance(
				diagram, false, false, english_class, englishChildrenBrothers);
		
		generalization_factory.createInstance(
				diagram, gentleman_class, latinLover_class);
		
		return diagram;
	}

	// Students Diagram
	public CCDDiagram buildStudentsDiagram() {
		CCDDiagram diagram = new CCDDiagram("StudentDiagram");
		CCDClass_Factory class_factory = new CCDClass_Factory();
		CCDOperation_Factory op_factory = new CCDOperation_Factory();
		CCDAttribute_Factory attribute_factory = new CCDAttribute_Factory();
		CCDAssociation_Factory association_factory = new CCDAssociation_Factory();
		CCDParameter_Factory parameter_factory = new CCDParameter_Factory();

		CCDClass student_class = class_factory.createInstance(diagram,
				"Student");
		CCDAttribute identification_code_attribute =
			attribute_factory.createInstance(
					diagram, student_class, CTBaseType.STRING_TYPE,
						"identificationCode");
		CCDAttribute telephone_attribute = attribute_factory.createInstance(
				diagram, student_class, CTBaseType.STRING_TYPE, "telephone");
		CCDAttribute year_attribute =
			attribute_factory.createInstance(diagram,
				student_class, CTBaseType.INTEGER_TYPE, "courseYear");

		CCDOperation average_grade_operation = op_factory.createInstance(
				diagram, "averageGrade", student_class);
		// The Return Type of average_grade_operation is Float
		CTType_isTypeOf_CTTypeDefined.addLink(
				CTBaseType.FLOAT_TYPE, average_grade_operation);
		CCDOperation number_of_exams_operation = op_factory.createInstance(
				diagram, "numberOfExams", student_class);
		// The Return Type of number_of_exams_operation is Integer
		CTType_isTypeOf_CTTypeDefined.addLink(
				CTBaseType.INTEGER_TYPE, number_of_exams_operation);
		CCDParameter year_of_course_parameter = parameter_factory
				.createInstance(diagram, number_of_exams_operation,
						student_class, "yearOfCourse");
		return diagram;
	}

	// Soldiers Diagram
	public CCDDiagram buildSoldiersDiagram() {
		CCDDiagram diagram = new CCDDiagram("SoldierDiagram");
		CCDClass_Factory class_factory = new CCDClass_Factory();
		CCDOperation_Factory op_factory = new CCDOperation_Factory();
		CCDAttribute_Factory attribute_factory = new CCDAttribute_Factory();
		CCDAssociation_Factory association_factory =
			new CCDAssociation_Factory();
		CCDParameter_Factory parameter_factory = new CCDParameter_Factory();

		CCDClass soldier_class = class_factory.createInstance(diagram,
				"Soldier");
		CCDAttribute identification_code_attribute = attribute_factory
				.createInstance(
						diagram, soldier_class, CTBaseType.STRING_TYPE,
						"identificationCode");
		CCDAttribute weapon_attribute = attribute_factory.createInstance(
				diagram, soldier_class, CTBaseType.STRING_TYPE, "weapon");
		CCDOperation honor_worthy_operation = op_factory.createInstance(
				diagram, "honorWorthy", soldier_class);

		// The Return type of honor_worthy_operation is Boolean
		CTType_isTypeOf_CTTypeDefined.addLink(
				CTBaseType.BOOLEAN_TYPE, honor_worthy_operation);

		CCDClass officer_class = class_factory.createInstance(diagram,
				"Officer");
		CCDAttribute years_of_service_attribute = attribute_factory
				.createInstance(diagram, officer_class,
						CTBaseType.INTEGER_TYPE, "yearsOfService");

		CCDClass non_commissioned_officer_class =
			class_factory.createInstance(diagram, "NonCommissioned");
		CCDAttribute licenses_attribute = attribute_factory.createInstance(
				diagram, officer_class, CTBaseType.INTEGER_TYPE, "licenses");

		CCDClass troop_soldier_class = class_factory.createInstance(diagram,
				"TroopSoldier");
		CCDAttribute merit_notes_attribute =
			attribute_factory.createInstance(
				diagram, officer_class,
				CTBaseType.INTEGER_TYPE, "merit_notes");

		TreeSet<CCDClass> soldier_subClasses = new TreeSet<CCDClass>();
		soldier_subClasses.add(officer_class);
		soldier_subClasses.add(non_commissioned_officer_class);
		soldier_subClasses.add(troop_soldier_class);

		CCDGeneralization soldiers_generalization =
			new CCDGeneralization_Factory()
				.createInstance(diagram, true, true, soldier_class,
						soldier_subClasses);

		return diagram;
	}

	// Bank Transfers Diagram
	public CCDDiagram buildBankTransfersDiagram() {
		CCDDiagram diagram = new CCDDiagram("BankDiagram");
		CCDClass_Factory class_factory = new CCDClass_Factory();
		CCDOperation_Factory op_factory = new CCDOperation_Factory();
		CCDAttribute_Factory attribute_factory = new CCDAttribute_Factory();
		CCDAssociation_Factory association_factory = new CCDAssociation_Factory();
		CCDParameter_Factory parameter_factory = new CCDParameter_Factory();
		CCDSpecialization_Factory specialization_factory = new CCDSpecialization_Factory();
		
		CCDClass money_transfer_class = class_factory.createInstance(diagram,
				"MoneyTransfer");
		CCDClass money_transfer_from_person_class = class_factory
				.createInstance(diagram, "MoneyTransferFromPerson");
		CCDClass private_bank_account_class = class_factory.createInstance(
				diagram, "PrivateBankAccount");
		CCDClass society_bank_account_class = class_factory.createInstance(
				diagram, "SocietyBankAccount");
		CCDClass bank_account_class = class_factory.createInstance(diagram,
				"BankAccount");

		CCDGeneralization bank_transfer_generalization =
			new CCDGeneralization_Factory()
				.createInstance(diagram, money_transfer_class,
						money_transfer_from_person_class);

		TreeSet<CCDClass> bank_account_subClasses = new TreeSet<CCDClass>();
		bank_account_subClasses.add(private_bank_account_class);
		bank_account_subClasses.add(society_bank_account_class);
		CCDGeneralization bank_account_generalization =
			new CCDGeneralization_Factory()
				.createInstance(diagram, true, true, bank_account_class,
						bank_account_subClasses);

		List<CCDAssociation_Factory.Role_TemporaryContainer> order_roles =
			new ArrayList<CCDAssociation_Factory.Role_TemporaryContainer>(2);
		order_roles.add(new CCDAssociation_Factory.Role_TemporaryContainer(
				new Multiplicity((short) 1, (short) 1), bank_account_class));
		order_roles.add(new CCDAssociation_Factory.Role_TemporaryContainer(
				new Multiplicity((short) 0, Multiplicity.UNBOUNDED),
				money_transfer_class));
		CCDAssociation_Factory.Association_Roles_Container
		transfer_container =
			association_factory.createInstances(
					diagram, "orders", order_roles);

		List<CCDAssociation_Factory.Role_TemporaryContainer>
		person_order_roles =
			new ArrayList<CCDAssociation_Factory.Role_TemporaryContainer>(2);
		person_order_roles
				.add(new CCDAssociation_Factory.Role_TemporaryContainer(
						private_bank_account_class));
		person_order_roles
				.add(new CCDAssociation_Factory.Role_TemporaryContainer(
						money_transfer_from_person_class));
		CCDAssociation_Factory.Association_Roles_Container
		person_order_container =
			association_factory.createInstances(
					diagram, "personOrders", person_order_roles);

		// person_orders is a subset of orders
		CCDSpecialization hatred_argue_specialization = specialization_factory
		.createInstance(diagram, transfer_container.association,
				person_order_container.association);

		return diagram;
	}

	// Hospital Diagram
	public CCDDiagram buildHospitalDiagram() {
		CCDDiagram diagram = new CCDDiagram("HospitalDiagram");
		CCDClass_Factory class_factory = new CCDClass_Factory();
		CCDOperation_Factory op_factory = new CCDOperation_Factory();
		CCDAttribute_Factory attribute_factory = new CCDAttribute_Factory();
		CCDAssociation_Factory association_factory =
			new CCDAssociation_Factory();
		CCDParameter_Factory parameter_factory = new CCDParameter_Factory();

		CCDClass person_class =
			class_factory.createInstance(diagram, "Person");
		CCDAttribute name_attribute =
			attribute_factory.createInstance(diagram,
				person_class, CTBaseType.STRING_TYPE, "name");
		CCDAttribute surname_attribute = attribute_factory.createInstance(
				diagram, person_class, CTBaseType.STRING_TYPE, "surname");
		CCDAttribute birth_date_attribute = attribute_factory.createInstance(
				diagram, person_class, CTBaseType.DATE_TYPE, "birthDate");
		CCDAttribute regional_code_attribute = attribute_factory
				.createInstance(diagram, person_class, CTBaseType.STRING_TYPE,
						"regional_code");

		CCDClass prescription_class = class_factory.createInstance(diagram,
				"Prescription");
		CCDAttribute code_attribute =
			attribute_factory.createInstance(diagram,
				prescription_class, CTBaseType.STRING_TYPE, "code");

		CCDClass medicine_class = class_factory.createInstance(diagram,
				"Medicine");
		CCDAttribute amount_attribute = attribute_factory.createInstance(
				diagram, medicine_class, CTBaseType.INTEGER_TYPE, "amount");

		CCDClass hospitalize_request_class = class_factory.createInstance(
				diagram, "HospitalizeRequest");
		CCDAttribute department_attribute = attribute_factory.createInstance(
				diagram, hospitalize_request_class, CTBaseType.STRING_TYPE,
				"department");

		CCDClass checkup_class = class_factory.createInstance(diagram,
				"Checkup");
		CCDAttribute perform_date_attribute = attribute_factory.createInstance(
				diagram, checkup_class, CTBaseType.DATE_TYPE, "performDate");
		CCDOperation is_performed_operation = op_factory.createInstance(
				diagram, "isPerformed", checkup_class);
		// The Return type of is_performed_operation is boolean
		CTType_isTypeOf_CTTypeDefined.addLink(CTBaseType.BOOLEAN_TYPE, is_performed_operation);
		
		// The Return type of insert_indication_operation is void
		CCDOperation insert_indication_operation = op_factory.createInstance(
				diagram, "insertIndication", checkup_class);
		
		CCDParameter prescription_parameter = parameter_factory.createInstance(
				diagram, insert_indication_operation, prescription_class,
				"prescription");

		TreeSet<CCDClass> prescription_subClasses = new TreeSet<CCDClass>();
		prescription_subClasses.add(checkup_class);
		prescription_subClasses.add(hospitalize_request_class);
		CCDGeneralization prescription_generalization = new CCDGeneralization_Factory()
				.createInstance(diagram, true, true, prescription_class,
						prescription_subClasses);

		List<CCDAssociation_Factory.Role_TemporaryContainer> patient_roles = new ArrayList<CCDAssociation_Factory.Role_TemporaryContainer>(
				2);
		patient_roles.add(new CCDAssociation_Factory.Role_TemporaryContainer(
				new Multiplicity((short) 1, (short) 1), person_class));
		patient_roles.add(new CCDAssociation_Factory.Role_TemporaryContainer(
				new Multiplicity((short) 0, Multiplicity.UNBOUNDED),
				checkup_class));
		CCDAssociation_Factory.Association_Roles_Container transfer_container = association_factory
				.createInstances(diagram, "patient", patient_roles);

		List<CCDAssociation_Factory.Role_TemporaryContainer> doctor_roles = new ArrayList<CCDAssociation_Factory.Role_TemporaryContainer>(
				2);
		doctor_roles.add(new CCDAssociation_Factory.Role_TemporaryContainer(
				new Multiplicity((short) 1, (short) 1), person_class));
		doctor_roles.add(new CCDAssociation_Factory.Role_TemporaryContainer(
				new Multiplicity((short) 0, Multiplicity.UNBOUNDED),
				checkup_class));
		CCDAssociation_Factory.Association_Roles_Container doctor_container = association_factory
				.createInstances(diagram, "doctor", doctor_roles);

		List<CCDAssociation_Factory.Role_TemporaryContainer> indication_roles = new ArrayList<CCDAssociation_Factory.Role_TemporaryContainer>(
				2);
		indication_roles
				.add(new CCDAssociation_Factory.Role_TemporaryContainer(
						new Multiplicity((short) 1, (short) 1),
						prescription_class));
		indication_roles
				.add(new CCDAssociation_Factory.Role_TemporaryContainer(
						new Multiplicity((short) 0, Multiplicity.UNBOUNDED),
						checkup_class));
		CCDAssociation_Factory.Association_Roles_Container indication_container = association_factory
				.createInstances(diagram, "indication", indication_roles);
		return diagram;
	}

	private String transform(CCDDiagram diagram) {
		return new XmlMarshaller().marshal(diagram);
	}

	public boolean test(String testCode) {
		CCDDiagram diagram =
			(testCode.equals(MainReDiAVeriFIntTest.ANIMALS_TEST_CODE) ?
				this.buildAnimalsDiagram() :
				(testCode.equals(MainReDiAVeriFIntTest.STUDENT_TEST_CODE) ?
					this.buildStudentsDiagram() :
					(testCode.equals(MainReDiAVeriFIntTest.SOLDIERS_TEST_CODE) ?
						this.buildSoldiersDiagram() :
						(testCode.equals(MainReDiAVeriFIntTest.BANK_TRANSFERS_TEST_CODE) ? 
							this.buildBankTransfersDiagram() :
							(testCode.equals(MainReDiAVeriFIntTest.HOSPITAL_TEST_CODE) ?
								this.buildHospitalDiagram() :
								(testCode.equals(MainReDiAVeriFIntTest.ENGLISH_ITALIANS_TEST_CODE) ?
									this.buildEnglishItaliansDiagram() :
									(testCode.equals(MainReDiAVeriFIntTest.DEFAULT_TEST_CODE) ?
										this.buildAnimalsDiagram() :
										null
									)
								)
							)
						)
					)
				)
			)
		;

		if (diagram == null) {
			return false;
		}

		this.xmlDiagram = this.transform(diagram);
		System.out.println(this.xmlDiagram);
		return true;
	}

	public String getXmlDiagram() {
		return this.xmlDiagram;
	}

	public void test() {
		this.test(MainReDiAVeriFIntTest.DEFAULT_TEST_CODE);
	}
}