package it.uniroma1.dis.mfis.redia.verifint.test.testers;

import it.uniroma1.dis.mfis.redia.verifint.test.MainReDiAVeriFIntTest;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Drawable;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Edge;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Figure;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Frame;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Label;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.RenderingPane;

import java.awt.Point;
import java.util.ArrayList;
import java.util.TreeSet;

import javax.swing.JFrame;

public class GfxTester {
	private RenderingPane renderingPane = new RenderingPane();
	
	public boolean test(String testCode) {
		if (!this.buildRenderingPane(testCode))
			return false;
		JFrame testFrame = new JFrame("Test Frame");
		testFrame.setContentPane(this.renderingPane);                
		testFrame.setVisible(true);
		testFrame.setBounds(140, 130, 640, 480);
		testFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		return true;
	}

	private boolean buildRenderingPane(String testCode) {
		TreeSet<Drawable> drawableObjects = 
			(testCode.equals(MainReDiAVeriFIntTest.STUDENT_TEST_CODE) ?
				this.drawStudentsDiagram() :
				(testCode.equals(MainReDiAVeriFIntTest.BANK_TRANSFERS_TEST_CODE) ?
					this.drawBankTransfersDiagram() :
					(testCode.equals(MainReDiAVeriFIntTest.SOLDIERS_TEST_CODE) ?
						this.drawSoldiersDiagram() :
						(testCode.equals(MainReDiAVeriFIntTest.HOSPITAL_TEST_CODE) ? 
							this.drawHospitalDiagram() :
							(testCode.equals(MainReDiAVeriFIntTest.ENGLISH_ITALIANS_TEST_CODE) ?
								this.drawLatinLoverDiagram() :
								(testCode.equals(MainReDiAVeriFIntTest.DEFAULT_TEST_CODE) ?
										this.drawHospitalDiagram() :
										null
									)
							)
						)
					)
				)
			)
		;
		if (drawableObjects == null)
			return false;
		this.renderingPane.setDrawableObjects(drawableObjects);
		return true;
	}

	public void test() {
		this.test(MainReDiAVeriFIntTest.DEFAULT_TEST_CODE);
	}

	private TreeSet<Drawable> drawStudentsDiagram() {
		TreeSet<Drawable> drawableObjects = new TreeSet<Drawable>();
		Frame student_frame = new Frame(new Point(200, 50));
		Label student_label = new Label("Student");
		student_label.setFont(RenderingPane.BOLD_FONT);
		student_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		student_frame.getFigure().add(student_label);
		student_frame.getFigure().add(new Figure());
		student_frame.getFigure().add(
				new Label("identification_code : String"));
		student_frame.getFigure().add(new Label("telephone : String"));
		student_frame.getFigure().add(
				new Label("course_year : Positive Integer"));
		student_frame.getFigure().add(new Figure());
		student_frame.getFigure().add(new Label("averageGrade() : Real"));
		student_frame.getFigure().add(
				new Label("numberOfExams(Integer) : String"));

		drawableObjects.add(student_frame);
		
		return drawableObjects;
	}

	private TreeSet<Drawable> drawLatinLoverDiagram()
	{
		TreeSet<Drawable> drawableObjects = new TreeSet<Drawable>();
		Frame person_frame = new Frame(new Point(200, 50));
		Label student_label = new Label("Person");
		student_label.setFont(RenderingPane.BOLD_FONT);
		student_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		person_frame.getFigure().add(student_label);
		drawableObjects.add(person_frame);

		Frame italian_frame = new Frame(new Point(100, 150));
		Label italian_label = new Label("Italian");
		italian_label.setFont(RenderingPane.BOLD_FONT);
		italian_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		italian_frame.getFigure().add(italian_label);
		drawableObjects.add(italian_frame);

		Frame english_frame = new Frame(new Point(300, 150));
		Label english_label = new Label("English");
		english_label.setFont(RenderingPane.BOLD_FONT);
		english_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		english_frame.getFigure().add(english_label);
		drawableObjects.add(english_frame);

		Frame lazy_frame = new Frame(new Point(20, 250));
		Label lazy_label = new Label("Lazy");
		lazy_label.setFont(RenderingPane.BOLD_FONT);
		lazy_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		lazy_frame.getFigure().add(lazy_label);
		drawableObjects.add(lazy_frame);

		Frame latinlover_frame = new Frame(new Point(110, 250));
		Label latinlover_label = new Label("LatinLover");
		latinlover_label.setFont(RenderingPane.BOLD_FONT);
		latinlover_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		latinlover_frame.getFigure().add(latinlover_label);
		drawableObjects.add(latinlover_frame);

		Frame gentleman_frame = new Frame(new Point(250, 250));
		Label gentleman_label = new Label("Gentleman");
		gentleman_label.setFont(RenderingPane.BOLD_FONT);
		gentleman_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		gentleman_frame.getFigure().add(gentleman_label);
		drawableObjects.add(gentleman_frame);

		Frame hooligan_frame = new Frame(new Point(350, 250));
		Label hooligan_label = new Label("Hooligan");
		hooligan_label.setFont(RenderingPane.BOLD_FONT);
		hooligan_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		hooligan_frame.getFigure().add(hooligan_label);
		drawableObjects.add(hooligan_frame);

		ArrayList person_subClasses = new ArrayList();
		person_subClasses.add(italian_frame);
		person_subClasses.add(english_frame);

		ArrayList italian_subClasses = new ArrayList();
		italian_subClasses.add(latinlover_frame);
		italian_subClasses.add(lazy_frame);

		ArrayList english_subClasses = new ArrayList();
		english_subClasses.add(gentleman_frame);
		english_subClasses.add(hooligan_frame);

		Edge person_hierarchy = new Edge(person_subClasses, person_frame);
		person_hierarchy.setStyle(Edge.STYLE_ISA);
		person_hierarchy.setLabelName("{disjoint}");
		drawableObjects.add(person_hierarchy);

		Edge italian_hierarchy = new Edge(italian_subClasses, italian_frame);
		italian_hierarchy.setStyle(Edge.STYLE_ISA);
		italian_hierarchy.setLabelName("{disjoint, covering}");
		drawableObjects.add(italian_hierarchy);

		Edge english_hierarchy = new Edge(english_subClasses, english_frame);
		english_hierarchy.setStyle(Edge.STYLE_ISA);
		drawableObjects.add(english_hierarchy);

		Edge latinlover_gentleman = new Edge(latinlover_frame, gentleman_frame);
		drawableObjects.add(latinlover_gentleman);
		
		return drawableObjects;
	}

	private TreeSet<Drawable> drawSoldiersDiagram() {
		TreeSet<Drawable> drawableObjects = new TreeSet<Drawable>();
		Frame soldier_frame = new Frame(new Point(200, 50));
		Label soldier_label = new Label("Soldier");
		soldier_label.setFont(RenderingPane.BOLD_FONT);
		soldier_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		soldier_frame.getFigure().add(soldier_label);
		soldier_frame.getFigure().add(new Figure());
		soldier_frame.getFigure().add(
				new Label("identification_code : String"));
		soldier_frame.getFigure().add(new Label("weapon : String"));
		soldier_frame.getFigure().add(new Figure());
		soldier_frame.getFigure().add(new Label("honorWorthy() : Boolean"));
		drawableObjects.add(soldier_frame);

		Frame officer_frame = new Frame(new Point(50, 250));
		Label officer_label = new Label("Officer");
		officer_label.setFont(RenderingPane.BOLD_FONT);
		officer_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		officer_frame.getFigure().add(officer_label);
		officer_frame.getFigure().add(new Figure());
		officer_frame.getFigure().add(new Label("years_of_service : Integer"));
		drawableObjects.add(officer_frame);

		Frame non_commissioned_frame = new Frame(new Point(230, 250));
		Label non_commissioned_label = new Label("NonCommissioned");
		non_commissioned_label.setFont(RenderingPane.BOLD_FONT);
		non_commissioned_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		non_commissioned_frame.getFigure().add(non_commissioned_label);
		non_commissioned_frame.getFigure().add(new Figure());
		non_commissioned_frame.getFigure().add(new Label("licenses : Integer"));
		drawableObjects.add(non_commissioned_frame);

		Frame troop_soldier_frame = new Frame(new Point(400, 250));
		Label troop_soldier_label = new Label("TroopSoldier");
		troop_soldier_label.setFont(RenderingPane.BOLD_FONT);
		troop_soldier_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		troop_soldier_frame.getFigure().add(troop_soldier_label);
		troop_soldier_frame.getFigure().add(new Figure());
		troop_soldier_frame.getFigure().add(new Label("merit_notes : Integer"));
		drawableObjects.add(troop_soldier_frame);

		ArrayList soldier_subClasses = new ArrayList();
		soldier_subClasses.add(officer_frame);
		soldier_subClasses.add(non_commissioned_frame);
		soldier_subClasses.add(troop_soldier_frame);

		Edge soldier_generalization_edge = new Edge(soldier_subClasses,
				soldier_frame);
		soldier_generalization_edge.setStyle(Edge.STYLE_ISA);
		soldier_generalization_edge.setLabelName("{disjoint, complete}");
		drawableObjects.add(soldier_generalization_edge);

		return drawableObjects;
	}

	private TreeSet<Drawable> drawBankTransfersDiagram() {
		TreeSet<Drawable> drawableObjects = new TreeSet<Drawable>();
		Frame money_transfer_frame = new Frame(new Point(100, 50));
		Label money_transfer_label = new Label("MoneyTransfer");
		money_transfer_label.setFont(RenderingPane.BOLD_FONT);
		money_transfer_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		money_transfer_frame.getFigure().add(money_transfer_label);
		money_transfer_frame.getFigure().add(new Figure());
		drawableObjects.add(money_transfer_frame);

		Frame money_transfer_from_person_frame = new Frame(new Point(20, 250));
		Label money_transfer_from_person_label = new Label(
				"MoneyTransferFromPerson");
		money_transfer_from_person_label.setFont(RenderingPane.BOLD_FONT);
		money_transfer_from_person_label.setTextAlignment(
				Label.ALIGNMENT_CENTER);
		money_transfer_from_person_frame.getFigure().add(
				money_transfer_from_person_label);
		money_transfer_from_person_frame.getFigure().add(new Figure());
		drawableObjects.add(money_transfer_from_person_frame);

		Frame private_bank_account_frame = new Frame(new Point(350, 250));
		Label private_bank_account_label = new Label("PrivateBankAccount");
		private_bank_account_label.setFont(RenderingPane.BOLD_FONT);
		private_bank_account_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		private_bank_account_frame.getFigure().add(private_bank_account_label);
		private_bank_account_frame.getFigure().add(new Figure());
		drawableObjects.add(private_bank_account_frame);

		Frame society_bank_account_frame = new Frame(new Point(480, 250));
		Label society_bank_account_label = new Label("SocietyBankAccount");
		society_bank_account_label.setFont(RenderingPane.BOLD_FONT);
		society_bank_account_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		society_bank_account_frame.getFigure().add(society_bank_account_label);
		society_bank_account_frame.getFigure().add(new Figure());
		drawableObjects.add(society_bank_account_frame);

		Frame bank_account_frame = new Frame(new Point(350, 50));
		Label bank_account_label = new Label("BankAccount");
		bank_account_label.setFont(RenderingPane.BOLD_FONT);
		bank_account_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		bank_account_frame.getFigure().add(bank_account_label);
		bank_account_frame.getFigure().add(new Figure());
		drawableObjects.add(bank_account_frame);

		ArrayList bank_account_subClasses = new ArrayList();
		bank_account_subClasses.add(private_bank_account_frame);
		bank_account_subClasses.add(society_bank_account_frame);

		Edge bank_account_generalization_edge = new Edge(
				bank_account_subClasses, bank_account_frame);
		bank_account_generalization_edge.setLabelName("{disjoint, complete}");
		bank_account_generalization_edge.setStyle(Edge.STYLE_ISA);
		drawableObjects.add(bank_account_generalization_edge);
		Edge money_transfer_isa = new Edge(money_transfer_from_person_frame,
				money_transfer_frame);
		money_transfer_isa.setLabelName("");
		money_transfer_isa.setStyle(Edge.STYLE_ISA);
		drawableObjects.add(money_transfer_isa);

		Edge ac_to_mt = new Edge(bank_account_frame, money_transfer_frame);
		ac_to_mt.setLabelName("orders");
		ac_to_mt.setStyle(Edge.STYLE_HAS);
		ac_to_mt.setMultiplicity(0, "1..1");
		ac_to_mt.setMultiplicity(1, "0..*");
		drawableObjects.add(ac_to_mt);

		Edge pac_to_pmt = new Edge(private_bank_account_frame,
				money_transfer_from_person_frame);
		pac_to_pmt.setLabelName("person_orders");
		pac_to_pmt.setStyle(Edge.STYLE_HAS);
		drawableObjects.add(pac_to_pmt);

		return drawableObjects;
	}

	private TreeSet<Drawable> drawHospitalDiagram() {
		TreeSet<Drawable> drawableObjects = new TreeSet<Drawable>();
		Frame person_frame = new Frame(new Point(10, 100));
		Label person_label = new Label("Person");
		person_label.setFont(RenderingPane.BOLD_FONT);
		person_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		person_frame.getFigure().add(person_label);
		person_frame.getFigure().add(new Figure());
		person_frame.getFigure().add(new Label("name : String"));
		person_frame.getFigure().add(new Label("surname : String"));
		person_frame.getFigure().add(new Label("birth_date : Date"));
		person_frame.getFigure().add(new Label("regional_code : String"));
		drawableObjects.add(person_frame);

		Frame checkup_frame = new Frame(new Point(250, 50));
		Label checkup_label = new Label("Checkup");
		checkup_label.setFont(RenderingPane.BOLD_FONT);
		checkup_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		checkup_frame.getFigure().add(checkup_label);
		checkup_frame.getFigure().add(new Figure());
		checkup_frame.getFigure().add(new Label("perform_date : Date"));
		checkup_frame.getFigure().add(new Figure());
		checkup_frame.getFigure().add(new Label("isPerformed() : Boolean"));
		checkup_frame.getFigure().add(
				new Label("insertIndication(p: Prescription) : Void"));
		drawableObjects.add(checkup_frame);

		Frame prescription_frame = new Frame(new Point(300, 250));
		Label prescription_label = new Label("Prescription");
		prescription_label.setFont(RenderingPane.BOLD_FONT);
		prescription_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		prescription_frame.getFigure().add(prescription_label);
		prescription_frame.getFigure().add(new Figure());
		prescription_frame.getFigure().add(new Label("code : String"));
		drawableObjects.add(prescription_frame);

		Frame medicine_frame = new Frame(new Point(200, 380));
		Label medicine_label = new Label("Medicine");
		medicine_label.setFont(RenderingPane.BOLD_FONT);
		medicine_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		medicine_frame.getFigure().add(medicine_label);
		medicine_frame.getFigure().add(new Figure());
		medicine_frame.getFigure().add(new Label("amount : PositiveInteger"));
		drawableObjects.add(medicine_frame);

		Frame hospitalize_request_frame = new Frame(new Point(350, 380));
		Label hospitalize_request_label = new Label("HospitalizeRequest");
		hospitalize_request_label.setFont(RenderingPane.BOLD_FONT);
		hospitalize_request_label.setTextAlignment(Label.ALIGNMENT_CENTER);
		hospitalize_request_frame.getFigure().add(hospitalize_request_label);
		hospitalize_request_frame.getFigure().add(new Figure());
		hospitalize_request_frame.getFigure().add(
				new Label("department : String"));
		drawableObjects.add(hospitalize_request_frame);

		Edge patient_edge = new Edge(person_frame, checkup_frame);
		patient_edge.setLabelName("patient");
		patient_edge.setMultiplicity(0, "1..1");
		patient_edge.setMultiplicity(1, "0..*");
		patient_edge.setStyle(Edge.STYLE_SIMPLEASSOCIATION);
		drawableObjects.add(patient_edge);

		Edge doctor_edge = new Edge(person_frame, checkup_frame);
		doctor_edge.setLabelName("doctor");
		doctor_edge.setMultiplicity(0, "1..1");
		doctor_edge.setMultiplicity(1, "0..*");
		doctor_edge.setStyle(Edge.STYLE_SIMPLEASSOCIATION);
		drawableObjects.add(doctor_edge);

		Edge indication_edge = new Edge(checkup_frame, prescription_frame);
		indication_edge.setLabelName("indication");
		indication_edge.setMultiplicity(0, "1..1");
		indication_edge.setMultiplicity(1, "0..*");
		indication_edge.setStyle(Edge.STYLE_SIMPLEASSOCIATION);
		drawableObjects.add(indication_edge);

		ArrayList prescription_subClasses = new ArrayList();
		prescription_subClasses.add(medicine_frame);
		prescription_subClasses.add(hospitalize_request_frame);
		Edge prescription_generalization = new Edge(prescription_subClasses,
				prescription_frame);
		prescription_generalization.setLabelName("{disjoint, complete}");
		prescription_generalization.setStyle(Edge.STYLE_ISA);
		drawableObjects.add(prescription_generalization);

		return drawableObjects;
	}
}