package it.uniroma1.dis.mfis.redia.verifint.test.testers;

import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProvingResult;
import it.uniroma1.dis.mfis.redia.verifint.control.io.XmlMarshaller;
import it.uniroma1.dis.mfis.redia.verifint.control.provers.analysis.classDiagram.CCDProver9Prover;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDDiagram;
import it.uniroma1.dis.mfis.redia.verifint.test.MainReDiAVeriFIntTest;

/**
 * Test class to demonstrate the usage of the system for proving properties of diagrams.
 */
public class PropertyProvingTester {
	public boolean test(String testCode) {
		if (!testCode.equals(MainReDiAVeriFIntTest.ENGLISH_ITALIANS_TEST_CODE)
				&& !testCode.equals(MainReDiAVeriFIntTest.DEFAULT_TEST_CODE))
			return false;

		// prepare the diagram to test
		CCDDiagram testDiagram = prepareEnglishItalianTestDiagram();
		
		// marshal the given diagram in XML
		String xmlString = new XmlMarshaller().marshal(testDiagram);
		
		/*	the formula to prove about the given diagram:
		 *	no latin lover can exist!	*/
		String testGoal = "all X (-EnglishAndItalians_LatinLover(X)).";
		
		// proving
		ProvingResult proof = new CCDProver9Prover(testDiagram).prove(testGoal);
		
		// from now on, the print the results
		System.out.println("\n" +
				"########################################" + "\n" +
				"####   Given the model:                #" + "\n" +
				"########################################" + "\n"
		);
		System.out.println(xmlString);
		System.out.println("\n" +
				"########################################" + "\n" +
				"####   Given the goal:                 #" + "\n" +
				"########################################" + "\n"
		);
		System.out.println(testGoal);
		System.out.println("\n" +
				"########################################" + "\n" +
				"####   The formula is:                 #" + "\n" +
				"########################################" + "\n"
		);
		System.out.println(proof.proven);
		System.out.println("\n" +
				"########################################" + "\n" +
				"####   The proof is:                   #" + "\n" +
				"########################################" + "\n"
		);
		System.out.println(proof.proof);
		
		return true;
	}
	
	/**
	 * Builds a sample diagram, taken from Prof. E. Franconi's idea.
	 * @return A sample diagram
	 */
	private CCDDiagram prepareEnglishItalianTestDiagram() {
		return new DiagramToXmlTester().buildEnglishItaliansDiagram();
	}

	public void test() {
		this.test(MainReDiAVeriFIntTest.DEFAULT_TEST_CODE);
	}
}