/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.util.ArrayList;
import java.util.List;


/**
 * Classe che rappresenta e gestisce una Cornice grafica. Essa e' rappresentata
 * da un rettangolo e piu' etichette e separatori la cui posizione dipende 
 * da essa. Per Inserire un separatore e' necessario aggiungere alla lista 
 * figure una figura preferibilmente vuota, per inserire del testo e' necessario
 * aggiungere un' etichetta.
 * @author Fuego
 */
public class Frame extends Figure{
    private List<Figure> figures;
    private Color fillColor;
    private Color borderColor;
    
    private int xoffset = 5; // margine orizzontale rispetto al bordo
    private int interline = 5; // in pixel
    
    /**
     * Crea una nuova istanza di Cornice in una data posizione
     * @param position posizione grafica della cornice
     */
    public Frame(Point position)
    {        
        super(position);
        this.fillColor = new Color(248, 240, 186);
        this.borderColor = Color.BLACK;
        figures = new ArrayList<Figure>();
    }
    
  
    @Override
    public void draw(Graphics2D g)
    {        
        dimension.height = interline; dimension.width = 0;        
        FontMetrics metrics;
        for(Figure f : figures)
        {                    
            if(f instanceof Label) {
                Label e = (Label) f;
                metrics = g.getFontMetrics(e.getFont());
                dimension.height+=metrics.getHeight() + interline;
                if(metrics.stringWidth(e.getText()) > 
                        dimension.width) 
                    dimension.width = metrics.stringWidth(
                            e.getText());
            } else {
                dimension.height+=interline * 2;
            }
        }        
        
        dimension.width+=(xoffset * 2); // Aggiungo un margine orizzontale      
        g.setColor(new Color(210, 210, 210));
        g.fillRect(position.x + 4, position.y + 4, dimension.width, 
                dimension.height);
        g.setColor(getFillColor());
        g.fillRect(position.x, position.y, dimension.width, 
                dimension.height);
        g.setColor(getBorderColor());
        g.drawRect(position.x, position.y, dimension.width, 
                dimension.height);
        
        // Posizioni dell'etichetta calcolate in base all'allineamento
        int px = 0, py = position.y; 
        for(Figure f : figures)
        {   
            if(f instanceof Label) {
                Label e = (Label) f;
                py = py + g.getFontMetrics(e.getFont()).getHeight();
                switch(e.getTextAlignment())
                {
                    case Label.ALIGNMENT_CENTER:
                        px = (dimension.width / 2) - 
                                (g.getFontMetrics(
                                e.getFont()).stringWidth(e.getText()) / 2) 
                                + position.x;                    
                    break;
                    case Label.ALIGNMENT_RIGHT:
                        px = dimension.width - 
                                g.getFontMetrics(e.getFont()).stringWidth(
                                e.getText()) + position.x - xoffset;                    
                    break;
                    case Label.ALIGNMENT_LEFT:
                    default:
                        px = position.x + xoffset;
                    break;
                }            
                e.setPosition(new Point(px, py));
                e.draw(g);
            } else {
                py+=interline;
                g.setColor(this.borderColor);
                f.position.x = this.position.x;
                f.position.y = py;
                f.dimension.width = this.dimension.width;
                f.dimension.height = 0;
                f.draw(g);
            }
            py+=interline;
            
        }
    }
    
    /**
     * Restituisce la lista delle figure (etichette e separatori noti come Figure)
     * all'interno della cornice.
     * @return lista delle figure contenute nella cornice
     */
    public List<Figure> getFigure()
    {
        return figures;
    }

    /**
     * Imposta il colore di riempimento della cornice
     * @param fillColor colore di riempimento della cornice
     */
    public void setFillColor(Color fillColor) {
        this.fillColor = fillColor;
    }

    /**
     * Imposta il colore di bordo della cornice
     * @param borderColor colore di bordo della cornice
     */
    public void setBorderColor(Color borderColor) {
        this.borderColor = borderColor;
    }
    
    /**
     * Restituisce il colore di riempimento attuale della cornice
     * @return colore di riempimento attuale della cornice
     */
    public Color getFillColor() {
        return fillColor;
    }

    
    /**
     * Restituisce il colore di bordo attuale della cornice
     * @return colore di bordo attuale della cornice
     */
    public Color getBorderColor() {
        return borderColor;
    }
    
    @Override
    public int compareTo(Object o)
    {
        if(this != o)
            return 1;
        else return 0;
    }
    
}
