/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Point;

/**
 * Rappresenta un elemento grafico testuale. Puo' avere alcuni elementi di
 * formattazione particolare quali tipo di carattere, posizione e allineamento
 * del testo nel caso sia contenuta all'interno di una cornice.
 * @author Fuego
 */
public class Label extends Figure {

    public static final int ALIGNMENT_CENTER = 0;
    public static final int ALIGNMENT_LEFT = 1;
    public static final int ALIGNMENT_RIGHT = 2;
    private boolean opaque = false;
    private String text;
    private Font font;
    private Color color;
    private int textAlign;
    

    /**
     * Crea un'istanza di etichetta a partire da una stringa
     * @param text Stringa contenuta nell'etichetta
     */
    public Label(String text) {
        super(new Point(0, 0));
        this.text = text;
        this.font = new Font("Sans Serif", Font.PLAIN, 12);
        this.color = Color.BLACK;
        textAlign = ALIGNMENT_LEFT;
    }

    /**
     * Restituisce la stringa contenuta nell'etichetta
     * @return stringa contenuta nell'etichetta
     */
    public String getText() {
        return text;
    }

    /**
     * Imposta una stringa all'etichetta
     * @param contenuto stringa da impostare all'etichetta
     */
    public void setText(String contenuto) {
        this.text = contenuto;
    }

    /**
     * Restituisce lo stile del carattere attuale dell'etichetta
     * @return Stile carattere attuale dell'etichetta
     */
    public Font getFont() {
        return font;
    }

    /**
     * Imposta uno stile di carattere all'etichetta
     * @param font stile di carattere da impostare
     */
    public void setFont(Font font) {
        this.font = font;
    }

    @Override
    public void draw(Graphics2D g) {
        Color vecchioColore = g.getColor();
        FontMetrics metrics = g.getFontMetrics();
        dimension = metrics.getStringBounds(
                this.text, g).getBounds().getSize();                
        
        g.setFont(font);
        if(opaque) {
            
            g.setColor(Color.WHITE);
            g.fillRect(position.x, position.y - metrics.getHeight(), 
                    dimension.width, dimension.height);
        }
        g.setColor(color);
        g.drawString(text, position.x, position.y);
        g.setColor(vecchioColore);
    }

    @Override
    public String toString() {
        return getText();
    }

    /**
     * Restituisce l'allineamento del testo nel caso l'etichetta venga associata
     * ad un oggetto contenitore come ad esempio una cornice.
     * @return allineamento del testo
     */
    public int getTextAlignment() {
        return textAlign;
    }

    /**
     * Restituisce l'allineamento del testo nel caso l'etichetta venga associata
     * ad un oggetto contenitore come ad esempio una cornice.
     * @param allineamentoTesto
     */
    public void setTextAlignment(int allineamentoTesto) {
        this.textAlign = allineamentoTesto;
    }
    
    /**
     * Imposta uno sfondo bianco all'etichetta
     * @param opaque true se voglio rendere l'etichetta opaca, false se voglio
     * renderla trasparente
     */
    public void setOpaque(boolean opaque) {
        this.opaque = opaque;
    }
    
    /**
     * Imposta il colore del testo dell'etichetta
     * @param color
     */
    public void setColor(Color color) {
        this.color = color;
    }

    @Override
    public int compareTo(Object o) {
        if (this != o) {
            return -1;
        } else {
            return 0;
        }
    }
}
