/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine;

import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Line2D;

/**
 * Rappresenta il punto di aggancio di una Figura. Esso e' definito da una
 * posizione relativa alla figura a cui e' agganciato e la figura alla quale
 * appartiene il punto.
 * @author Fuego
 */
public class LinkPoint implements Drawable {

    private Point position; //Posizione dell'aggancio che E' SEMPRE relativa alla figura!
    private Figure figure;
    private Label multiplicityLabel;
    private final int UP = 0;
    private final int LEFT = 1;
    private final int RIGHT = 2;
    private final int DOWN = 3;
    private int side = UP;
    
    private final int labelOffset = 6;

    /**
     * Crea un'istanza del punto di aggancio data una figura e una posizione.
     * La posizione e' qualsiasi e verra' automaticamente agganciata alla figura
     * data.
     * @param figure Figura da associare al punto di aggancio
     * @param position Posizione del punto di riferimento in base al quale verra'
     * calcolata la posizione del punto di aggancio
     * @param multiplicity cardinalita' del punto di aggancio
     */
    public LinkPoint(Figure figure, Point position, String multiplicity) {
        this.figure = figure;
        // Il punto di aggancio deve essere SEMPRE sul bordo della figura
        if (position != null) {
            hookPoint(position);
        }
        this.multiplicityLabel = new Label(multiplicity);
        this.multiplicityLabel.setFont(new Font("Sans Serif", Font.PLAIN, 12));
        this.multiplicityLabel.setOpaque(true);
        this.makeRelativePosition();
    }

    /**
     * Rende la posizione relativa alla figura e non assoluta (per comodita' di
     * codifica)
     */
    private void makeRelativePosition() {
        if (position != null) {
            this.position.x -= figure.getPosition().x;
            this.position.y -= figure.getPosition().y;
        }
    }

    /**
     * Restituisce la posizione x di disegno del punto di aggancio
     * @return posizione x di disegno del punto di aggancio
     */
    public int getAbsoluteX() {
        return position.x + getFigure().getPosition().x;
    }

    /**
     * Restituisce la posizione y di disegno del punto di aggancio
     * @return posizione y di disegno del punto di aggancio
     */
    public int getAbsoluteY() {
        return position.y + getFigure().getPosition().y;
    }

    /**
     * Sceglie il punto di aggancio sul bordo della figura piu' vicino al punto
     * di riferimento dato.
     * @param p Punto di riferimento dell'aggancio
     */
    public void hookPoint(Point p) {
        position = getFigure().hookPoint(p);
        Rectangle rectangle = figure.getBoundingRect();
        Line2D lineUp = new Line2D.Float(new Point(rectangle.x, rectangle.y),
                new Point(rectangle.x + rectangle.width,
                rectangle.y));
        Line2D lineLeft = new Line2D.Float(new Point(rectangle.x, rectangle.y),
                new Point(rectangle.x,
                rectangle.y + rectangle.height));
        Line2D lineDown = new Line2D.Float(new Point(rectangle.x, rectangle.y +
                rectangle.height), new Point(rectangle.x +
                rectangle.width, rectangle.y + rectangle.height));
        Line2D lineRight = new Line2D.Float(new Point(rectangle.x +
                rectangle.width, rectangle.y + rectangle.height),
                new Point(rectangle.x + rectangle.width,
                rectangle.y));
        double distUp = lineUp.ptLineDist(position.x, position.y);
        double distDown = lineDown.ptLineDist(position.x, position.y);
        double distLeft = lineLeft.ptLineDist(position.x, position.y);
        double distDes = lineRight.ptLineDist(position.x, position.y);
        // Trovo il lato in cui il punto e' agganciato
        if (distUp <= distDown && distUp <= distLeft && distUp <= distDes) {
            side = UP;
        } else if (distDown <= distUp && distDown <= distLeft && distDown <= distDes) {
            side = DOWN;
        } else if (distDes <= distDown && distDes <= distLeft && distDes <= distUp) {
            side = RIGHT;
        } else if (distLeft <= distDown && distLeft <= distUp && distLeft <= distDes) {
            side = LEFT;
        }        
        makeRelativePosition();
    }

    /**
     * Restituisce un oggetto di tipo Point che rappresenta la posizione
     * assoluta del punto di aggancio
     * @return Point assoluto della posizione del punto di aggancio
     */
    public Point getPoint() {
        return new Point(getAbsoluteX(), getAbsoluteY());
    }

    /**
     * Restituisce la figura associata al punto di aggancio
     * @return Figura associata al punto di aggancio
     */
    public Figure getFigure() {
        return figure;
    }

    /**
     * 
     * @param multiplicity      
     */
    public void setMultiplicity(String multiplicity) {
        multiplicityLabel.setText(multiplicity);
    }
    
    

    /**
     * 
     */
    private void calculateMultiplicityLabelPosition() {
        Point nuovaPosizione = new Point(getAbsoluteX(), getAbsoluteY());
        switch (side) {
            case UP:
                nuovaPosizione.y-=labelOffset;
                break;
            case DOWN:
                nuovaPosizione.y+=(multiplicityLabel.getDimension().height + labelOffset);                
                break;                
            case LEFT:
                nuovaPosizione.x-=
                (multiplicityLabel.getDimension().width + labelOffset);
                nuovaPosizione.y-=1;
                break;
            case RIGHT:
                nuovaPosizione.x+=labelOffset;
                nuovaPosizione.y-=1;
                break;
        }
        multiplicityLabel.setPosition(nuovaPosizione);
    }

    public void draw(Graphics2D g) {
        calculateMultiplicityLabelPosition();
        multiplicityLabel.draw(g);
    }
}
