/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine;

import java.awt.Graphics2D;
import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

/**
 * Rappresenta una linea spezzata con un minimo di due vertici e il massimo 
 * ideale di infiniti vertici. 
 * @author Fuego
 */
public class Multiline implements Drawable {

    private Point start;
    private List<Point> vertices;

    /**
     * Crea una nuova linea spezzata come linea semplice, specificando il punto
     * di inizio e il punto di fine
     * @param start Punto di inizio della spezzata
     * @param end Punto di fine della spezzata
     */
    public Multiline(Point start, Point end) {
        this.start = start;
        ArrayList<Point> verticesList = new ArrayList();
        verticesList.add(end);
        this.vertices = verticesList;
    }

    /**
     * Crea una nuova linea spezzata con un vertice iniziale e una serie di
     * vertici seguenti
     * @param start Punto di inizio della spezzata
     * @param vertices Lista dei vertici seguenti
     */
    public Multiline(Point start, List<Point> vertices) {
        this.start = start;
        this.vertices = vertices;
    }

    public void draw(Graphics2D g) {
        int cx = getStart().x, cy = getStart().y;
        for (Point p : getVertices()) {
            g.drawLine(cx, cy, p.x, p.y);
            cx = p.x;
            cy = p.y;
        }
    }

    /**
     * Restituisce il punto iniziale della spezzata
     * @return Punto iniziale della spezzata
     */
    public Point getStart() {
        return start;
    }
        
    
    /**
     * Imposta il punto iniziale della spezzata
     * @param inizio Nuovo punto iniziale della spezzata
     */
    public void setStart(Point inizio) {
        this.start = inizio;
    }

    /**
     * Restituisce il punto finale della spezzata
     * @return Punto finale della spezzata
     */
    public Point getEnd() {
        return vertices.get(vertices.size());
    }

    
    /**
     * Restituisce la lista dei vertici che compongono la spezzata, in scrittura
     * e lettura
     * @return Lista dei vertici che compongono la spezzata
     */
    public List<Point> getVertices() {
        return vertices;
    }
}
