package it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;

import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.Set;
import java.util.TreeSet;
import javax.swing.JPanel;

/**
 * Pannello dove verrà disegnato il contenuto del database 
 * @author Fuego
 */
public class RenderingPane extends JPanel implements MouseListener,
		MouseMotionListener {

	public static final Font PLAIN_FONT = new Font("Sans Serif", Font.PLAIN, 12);
	public static final Font BOLD_FONT = new Font("Sans Serif", Font.BOLD, 12);
	public static final Font ITALIC_FONT = new Font("Sans Serif", Font.ITALIC, 12);

	private Set<Drawable> drawableObjects;
	private int gridThickness = 10;
	private boolean snappedToGrid = true;

	public RenderingPane() {
		super();
		this.setBackground(Color.WHITE);
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		this.setDrawableObjects(new TreeSet<Drawable>());

		/* Codice di test */
		/*
		Frame corn = new Frame(new Point(50, 60));
		Label e1 = new Label("<<Interface>>");
		e1.setTextAlignment(Label.ALIGNMENT_CENTER);
		e1.setFont(fontCorsivo);
		Label e2 = new Label("Disegnabile");
		e2.setTextAlignment(Label.ALIGNMENT_CENTER);
		e2.setFont(fontGrassetto);
		corn.getFigure().add(e1);
		corn.getFigure().add(e2);
		corn.getFigure().add(new Figure());
		corn.getFigure().add(new Label(""));
		corn.getFigure().add(new Figure());
		Label e3 = new Label("+disegna(g: Graphics2D): Void");
		corn.getFigure().add(e3);

		Frame corn2 = new Frame(new Point(330, 160));                       
		Label e4 = new Label("Figura");
		e4.setTextAlignment(Label.ALIGNMENT_CENTER);
		e4.setFont(fontGrassetto);
		corn2.getFigure().add(e4);        
		corn2.getFigure().add(new Figure());
		corn2.getFigure().add(new Label("-posizione: Point"));
		corn2.getFigure().add(new Label("-dimensione: Dimension"));
		corn2.getFigure().add(new Figure());
		corn2.getFigure().add(new Label("+getPosizione(): Point"));
		corn2.getFigure().add(new Label("+getDimensioni(): Dimension"));
		corn2.getFigure().add(new Label("+setPosizione(posizione: Point): Void"));
		corn2.getFigure().add(new Label("+setDimensioni(dimensioni: Point): Void"));
		corn2.getFigure().add(new Label("+getRettangoloContenitore(): Rectangle"));
		corn2.getFigure().add(new Label("+getBaricentro: Point"));
		corn2.getFigure().add(new Label("+disegna(g: Graphics2D): Void"));
		corn2.getFigure().add(new Label("+compareTo(o: Object): int"));
		
		
		drawableObjects.add(corn);
		drawableObjects.add(corn2);
		Frame corn3 = new Frame(new Point(400, 50));
		corn3.getFigure().add(new Label("Cornice 3"));
		corn3.getFigure().add(new Figure());
		corn3.getFigure().add(new Label("Contenuto"));        
		drawableObjects.add(corn3);
		Frame corn4 = new Frame(new Point(200, 150));
		corn4.getFigure().add(new Label("Cornice 4"));
		corn4.getFigure().add(new Figure());
		corn4.getFigure().add(new Label("Contenuto"));        
		drawableObjects.add(corn4);

		ArrayList lista = new ArrayList();
		lista.add(corn2);
		lista.add(corn3);
		lista.add(corn4);
		
		
		arco = new Edge(lista, corn);
		drawableObjects.add(arco);
		*/
	}

	public Set<Drawable> getDrawableObjects() {
		return this.drawableObjects;
	}

	public void setDrawableObjects(Set<Drawable> drawableObjects) {
		this.drawableObjects = drawableObjects;
	}

	public boolean isSnappedToGrid() {
		return snappedToGrid;
	}

	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2d = (Graphics2D) g;
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		//disegnaGriglia(g2d);
		g2d.drawString("Please left click to update edge positions.", 2, 15);
		for (Drawable d : drawableObjects)
			d.draw(g2d);
	}

	private void drawGrid(Graphics2D g) {
		Color oldColor = g.getColor();
		g.setColor(Color.GRAY);
		int horizontalNodesNumber = this.getWidth() / gridThickness,
			verticalNodesNumber = this.getHeight() / gridThickness;

		for (int j = 0; j <= verticalNodesNumber; j++)
			for (int i = 0; i <= horizontalNodesNumber; i++)
				g.fillRect(i * gridThickness, j * gridThickness, 1, 1);
		g.setColor(oldColor);
	}

	public void snapObjectsToGrid() {
		if (snappedToGrid)
			for (Drawable d : drawableObjects) {
				if (d instanceof Figure) {
					Figure f = (Figure) d;
					int x = (f.getPosition().x / gridThickness) * gridThickness;
					int y = (f.getPosition().y / gridThickness) * gridThickness;
					f.setPosition(new Point(x, y));
				}
			}
	}

	public int getGridThickness() {
		return gridThickness;
	}

	public void setGridThickness(int gridThickness) {
		this.gridThickness = gridThickness;
	}

	public void setSnappedToGrid(boolean snappedToGrid) {
		this.snappedToGrid = snappedToGrid;
	}

	public void mouseClicked(MouseEvent e) {
		snapObjectsToGrid();
		if (e.getButton() == MouseEvent.BUTTON3) {
			for (Drawable o : drawableObjects) {
				if (o instanceof Edge) {
					Edge a = (Edge) o;
					a.optimize();
				}
			}

		}
		this.repaint();
	}

	public void mousePressed(MouseEvent e) {

	}

	public void mouseReleased(MouseEvent e) {

	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {

	}

	public void mouseDragged(MouseEvent e) {
		int indiceArco = -1;
		for (Drawable o : drawableObjects) {
			if (o instanceof Edge) {
				Edge a = (Edge) o;
				for (int i = 0; i < a.getLinkPointCount(); i++) {
					indiceArco = a.getNearestLinkPoint(new Point(e.getX(), e
							.getY()));
					if (indiceArco >= 0) {
						a.setLinkPointPosition(indiceArco, e.getX(), e.getY());
						repaint();
					}
				}
			}
			if (indiceArco >= 0)
				break;
			if (o instanceof Figure) {
				Figure f = (Figure) o;

				if (f.getBoundingRect().contains(e.getX(), e.getY())) {
					Point p = new Point(e.getX() - f.getDimension().width / 2,
							e.getY() - f.getDimension().height / 2);
					p.x = (p.x / gridThickness) * gridThickness;
					p.y = (p.y / gridThickness) * gridThickness;
					f.setPosition(p);
					repaint();
				}
			}
		}
	}

	public void mouseMoved(MouseEvent e) {

	}
}