package it.uniroma1.dis.mfis.redia.verifint.view.widgets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;

import it.uniroma1.dis.mfis.redia.verifint.view.widgets._interfaces.WidgetListener;


public abstract class IOWidget extends Widget {
	private List<WidgetEventInterceptor> widgetEventInterceptors =
		new ArrayList<WidgetEventInterceptor>();
	protected String id;
	protected Object value;
	protected String action;

	public IOWidget(String label, String id, Object value, String action) {
		super(label);
		this.id = id;
		this.value = value;
		this.action = action;
	}

	public IOWidget(String label, String id, Object value) {
		super(label);
		this.id = id;
		this.value = value;
	}

	public IOWidget(String label, String id) {
		super(label);
		this.id = id;
	}

	public IOWidget() {
		super();
	}

	public void setId(String id) {
		this.id = id;
	}

	public void setValue(Object value) {
		this.value = value;
	}

	public String getId() {
		return id;
	}

	public Object getValue() {
		return value;
	}

	public String getAction() {
		return action;
	}

	public void setAction(String azione) {
		this.action = azione;
	}

	public void addWidgetEventInterceptors(WidgetListener w) {
		this.widgetEventInterceptors.add(new WidgetEventInterceptor(w, this));
	}

	public List<WidgetEventInterceptor> getWidgetEventInterceptors() {
		return widgetEventInterceptors;
	}

	protected static class WidgetEventInterceptor
	implements ActionListener, FocusListener {
		private WidgetListener widgetListener;
		private IOWidget widget;

		public WidgetEventInterceptor(
				WidgetListener widgetListener, IOWidget widget) {
			super();
			this.widgetListener = widgetListener;
			this.widget = widget;
		}

		public void actionPerformed(ActionEvent e) {
			this.widgetListener.widgetValueChanged(
					new WidgetEvent(
							e.getSource(),
							null,	// TODO dovrai metterci l'action!
							widget.getId(),
							widget.getValue()
						)
				);
		}

		public void focusGained(FocusEvent e) {}

		public void focusLost(FocusEvent e) {
			this.widgetListener.widgetValueChanged(
					new WidgetEvent(
							e.getSource(),
							null,	// TODO dovrai metterci l'action!
							widget.getId(),
							widget.getValue()
						)
				);
		}
	}
	
	public static class WidgetEvent extends EventObject {
		private static final long serialVersionUID = -4791623915152915904L;
		private String action;
		private String id;
		private Object value;

		private WidgetEvent(Object source, String action, String id,
				Object value) {
			super(source);
			this.action = action;
			this.id = id;
			this.value = value;
		}

		public WidgetEvent(Object source, String action, String id) {
			this(source, action, id, null);
		}

		public String getAction() {
			return action;
		}

		public String getId() {
			return id;
		}

		public Object getValue() {
			return value;
		}

		protected void setAction(String action) {
			this.action = action;
		}

		protected void setId(String id) {
			this.id = id;
		}

		protected void setValue(Object value) {
			this.value = value;
		}
	}
}