package it.uniroma1.dis.mfis.redia.verifint.view.widgets;

import java.awt.Component;
import java.awt.FlowLayout;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;


public class TextField extends Text {
	public static final String DEFAULT_ALL_CONTENT_ALLOWED = "";
	public String contentCheckRegularExpression = DEFAULT_ALL_CONTENT_ALLOWED;
	
	protected static class CheckedContentTextField extends JTextField {
		private static final long serialVersionUID = 6115818094560791322L;
		private String contentCheckRegularExpression;
		
		public CheckedContentTextField(String contentCheckRegularExpression) {
			super();
			this.contentCheckRegularExpression = contentCheckRegularExpression;
		}

		public CheckedContentTextField(
				String contentCheckRegularExpression, String content) {
			super(content);
			this.contentCheckRegularExpression = contentCheckRegularExpression;
		}
		
		protected Document createDefaultModel() {
			return new CheckedContentDocument(contentCheckRegularExpression);
		}
		
		private class CheckedContentDocument extends PlainDocument {
			private static final long serialVersionUID = -3818709137815002485L;
			private String contentCheckRegularExpression;

			public CheckedContentDocument(String contentCheckRegularExpression) {
				this.contentCheckRegularExpression = contentCheckRegularExpression;
			}

			public void insertString(int offs, String str, AttributeSet a)
				throws BadLocationException
			{
				if (str == null)
					return;

				CharSequence inputStr = str;
				Pattern pattern = Pattern.compile(this.contentCheckRegularExpression);
				Matcher matcher = pattern.matcher(inputStr);
				
				if(matcher.matches()){
					super.insertString(offs, str, a);
				}
				return;
			}
		}
	}

	@Override
	public Component generateGraphicComponent() {
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		JTextField textField = null;
		
		if (this.contentCheckRegularExpression.equals(DEFAULT_ALL_CONTENT_ALLOWED)) {
			textField = new CheckedContentTextField(contentCheckRegularExpression);
		} else {
			textField = new JTextField();
		}
		
		if (this.label != null) {
			panel.add(new JLabel(this.label));
		}
		
		if (this.value != null) {
			textField.setText(this.value.toString());
		}
		
		for (WidgetEventInterceptor interceptor :
			super.getWidgetEventInterceptors()) {
			textField.addFocusListener(interceptor);
			textField.addActionListener(interceptor);
		}

		// TODO add the Listeners!
		
		panel.add(textField);
		
		return panel;
	}
}